C ============================================================= C

      SUBROUTINE MEGCOI

C ============================================================= C
C This subroutine contains the definition of the COIL structure C
C ============================================================= C

C ============================================================= C
C This is a preliminary version of the coil design: we put  one C
C centimeter of aluminium coil into a cryostat 2.5 millimeters  C
C thick. We think this is a reasonable starting point. Whenever C
C we see at least a design in which this thickness can be       C
C reduced we will implement those new measures in the file.     C
C ============================================================= C

      IMPLICIT NONE
      
      CALL COIMAT
      CALL COIMED
      CALL COIVOL
      CALL COIPOS

      RETURN
      END

C ============================================================= C

      SUBROUTINE COIMAT

C ============================================================= C
C Coil materials were already defined; so this routine is       C
C completeley dummy and it was created only for uniformity.     C
C If such materials will be changed, the appropriate routine    C
C will be written.                                              C
C ============================================================= C

      IMPLICIT NONE

      RETURN
      END

C ============================================================= C

      SUBROUTINE COIMED

C ============================================================= C
C Coil media.                                                   C
C ============================================================= C

      IMPLICIT NONE

      INTEGER       NUMED,NATMED,ISVOL,IFIELD,IUPD,ISTPAR,NUMOLD
      REAL          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,CFIELD,PREC

C Presence of magnetic field in the tracking medium.  
      IFIELD = 1

C Maximum magnetic field. 
      FIELDM = 15.

C Maximum angular deviation.  
      TMAXFD = 20.0  

C Maximum step permitted (cm).
      STEMAX = 0.5

C Maximum fractional energy loss.
      DEEMAX = 0.01               

C Boundary crossing precision.
      EPSIL = 0.002 

C Minimum value for the maximum step imposed by dedx/mult scatt etc.
      STMIN = 0.001
                                                                 
C ============================================================= C
C Definitions of tracking media.                                C
C ============================================================= C
   
C Tracking medium 51.
      CALL GSTMED(51,'Coil Aluminium     $', 9,                               
     +       1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

C Tracking medium 52.
      CALL GSTMED(52,'Coil Vacuum        $',16,                               
     +      -1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

      RETURN
      END

C ============================================================= C

      SUBROUTINE COIVOL

C ============================================================= C
C Definition of coil volumes.                                   C
C ============================================================= C
      IMPLICIT NONE

      INTEGER IVOLU

      REAL CPAR(21)
      DATA CPAR/0.,360.,6.,125.,50., 51., 30., 50., 51., 20., 40.,
     +     41., -20., 40., 41., -30., 50., 51., -125., 50., 51./
      REAL IPAR(21)
      DATA IPAR/0.,360.,6.,125.,47., 47.25, 30., 47., 47.25, 20., 37.,
     +     37.25, -20., 37., 37.25, -30., 47., 47.25, -125., 47., 47.25/
      REAL FPAR(21)
      DATA FPAR/0.,360.,6.,125.,46.9, 54.1, 30., 46.9, 54.1, 20., 36.9,
     +     44.1, -20., 36.9, 44.1, -30., 46.9, 54.1, -125., 46.9, 54.1/
      REAL OPAR(21)
      DATA OPAR/0.,360.,6.,125.,53.75, 54., 30., 53.75, 54., 20., 43.75,
     +     44., -20., 43.75, 44., -30., 53.75, 54., -125., 53.75, 54./

C Center COIL
      REAL CCCO(3)
      DATA CCCO/35., 36.,   12./
      REAL CCIW(3)
      DATA CCIW/32., 32.25, 18./
      REAL CCOW(3)
      DATA CCOW/53.75, 54., 127./

C MID COIL
      REAL CMCO(3)
      DATA CMCO/40.5, 41.5,  5.5/
      REAL CMIW(3)
      DATA CMIW/36., 36.25,  7.5/

C Side COIL
      REAL CSCO(3)
      DATA CSCO/46., 47.,   47/
      REAL CSIW(3)
      DATA CSIW/42., 42.25, 47/

C Compensating coil
      REAL CASC(3)
      DATA CASC/120., 121.,   6./

C Coil FRAME
      REAL CFRA(3)
      DATA CFRA/32.,   54., 130./

C Coil made with policones

C      CALL GSVOLU('CFRA','PCON',52,FPAR,21,IVOLU)
C      CALL GSVOLU('COIL','PCON',51,CPAR,21,IVOLU)
C      CALL GSVOLU('CIWA','PCON',51,IPAR,21,IVOLU)
C      CALL GSVOLU('COWA','PCON',51,OPAR,21,IVOLU)

C Dummy coil

C      CALL GSVOLU('CDUM','TUBE',51,DUM1, 3,IVOLU)

C New volumes

      CALL GSVOLU('CFRA','TUBE',52,CFRA, 3,IVOLU)
C      CALL GSVOLU('CCVA','TUBE',52,CCVA, 3,IVOLU)
C      CALL GSVOLU('CSVA','TUBE',52,CSVA, 3,IVOLU)
      CALL GSVOLU('CCCO','TUBE',51,CCCO, 3,IVOLU)
      CALL GSVOLU('CCIW','TUBE',51,CCIW, 3,IVOLU)
      CALL GSVOLU('CCOW','TUBE',51,CCOW, 3,IVOLU)
      CALL GSVOLU('CSCO','TUBE',51,CSCO, 3,IVOLU)
      CALL GSVOLU('CSIW','TUBE',51,CSIW, 3,IVOLU)
      CALL GSVOLU('CMCO','TUBE',51,CMCO, 3,IVOLU)
      CALL GSVOLU('CMIW','TUBE',51,CMIW, 3,IVOLU)
      CALL GSVOLU('CASC','TUBE',51,CASC, 3,IVOLU)

      RETURN
      END

C ============================================================= C

      SUBROUTINE COIPOS

C ============================================================= C
C Coil volumes positioning.                                     C
C ============================================================= C

      IMPLICIT NONE

C Put the Outer Wall in the CFRA.
C      CALL GSPOS('COWA',1,'CFRA',0.,0.,0.,0,'ONLY')

C Put the Inner Wall in the CFRA.
C      CALL GSPOS('CIWA',1,'CFRA',0.,0.,0.,0,'ONLY')

C Put the COIL in the CFRA.
C      CALL GSPOS('COIL',1,'CFRA',0.,0.,0.,0,'ONLY')

C Put CFRA in the FRAM.
C      CALL GSPOS('CFRA',1,'FRAM',0.,0.,0.,0,'ONLY')

C Put DUMMY Coil (cilinder)
C      CALL GSPOS('CDUM',1,'FRAM',0.,0.,0.,0,'ONLY')

C Build central section
      CALL GSPOS('CCIW',1,'CFRA',0.,0.,0.,0,'ONLY')
      CALL GSPOS('CCOW',1,'CFRA',0.,0.,0.,0,'ONLY')
      CALL GSPOS('CCCO',1,'CFRA',0.,0.,0.,0,'ONLY')

C Build MID section
C Z.GT.0
      CALL GSPOS('CMIW',1,'CFRA',0.,0., 25.5,0,'ONLY')
      CALL GSPOS('CMCO',1,'CFRA',0.,0., 23.5,0,'ONLY')

C Z.LT.0
      CALL GSPOS('CMIW',2,'CFRA',0.,0.,-25.5,0,'ONLY')
      CALL GSPOS('CMCO',2,'CFRA',0.,0.,-23.5,0,'ONLY')

C Build side section
C Z.GT.0
      CALL GSPOS('CSIW',1,'CFRA',0.,0., 80.,0,'ONLY')
      CALL GSPOS('CSCO',1,'CFRA',0.,0., 78.,0,'ONLY')

C Z.LT.0
      CALL GSPOS('CSIW',2,'CFRA',0.,0.,-80.,0,'ONLY')
      CALL GSPOS('CSCO',2,'CFRA',0.,0.,-78.,0,'ONLY')

C Build ACTIVE SHIELD
C Z.GT.0
      CALL GSPOS('CASC',1,'CFRA',0.,0., 120.,0,'ONLY')
C      CALL GSPOS('CSCO',1,'CFRA',0.,0., 78.,0,'ONLY')

C Z.LT.0
      CALL GSPOS('CASC',2,'CFRA',0.,0.,-120.,0,'ONLY')
C      CALL GSPOS('CSCO',2,'CFRA',0.,0.,-78.,0,'ONLY')

C Build solenoid
      CALL GSPOS('CFRA',1,'FRAM',0.,0.,  0.0,0,'ONLY')
C      CALL GSPOS('CSVA',1,'FRAM',0.,0., 77.5,0,'ONLY')
C      CALL GSPOS('CSVA',2,'FRAM',0.,0.,-77.5,0,'ONLY')
      RETURN
      END
















