
      SUBROUTINE MEGEVE(ITYPE,CODE,VSTART,PSTART)

C ================================================================= C
C Generate events according to ITYPE.                               C
C ================================================================= C
                                                                   
C ================================================================= C
C ITYPE     N.   : Description                             MNEMONIC C
C ================================================================= C 
C ITYPE  =  1    : e+ gamma SIGNAL  event;                  SIGNAL  C
C ================================================================= C 
C ITYPE  =  2    : e+ gamma Radiative Decay                 RADDEC  C
C ITYPE  =  3    : e+ from MICHEL, gamma from RD            MICHRD  C
C ITYPE  =  4    : e+ from MICHEL, gamma from AIF           MICHAF  C
C ITYPE  =  5..9 : free for pair events;                            C
C ================================================================= C 
C ITYPE  = 11    : e+ from SIGNAL                           SIGNAE  C
C ITYPE  = 12    : e+ from MICHEL                           MICHEE  C
C ITYPE  = 13    : e+ flat noise on spectrometer            NOISEE  C
C ITYPE  = 14..19: free for single e+ events                        C
C ================================================================= C 
C ITYPE  = 21    : gamma from SIGNAL                        SIGNAG  C
C ITYPE  = 22    : gamma from RD                            RDGAMA  C
C ITYPE  = 23    : gamma from AIF                           AFGAMA  C
C ITYPE  = 24    : gamma flat                               FLGAMA  C
C ITYPE  = 25    : pile-up between RD gamma's               PILRAD  C
C ITYPE  = 26    : pile-up between AIF gamma's              PILAIF  C
C ITYPE  = 27    : pile-up between RD+AIF gamma's           PILRAI  C
C ITYPE  = 28    : pile-up between flat spectrum gamma's    PILFLA  C
C ITYPE  = 29: free for single gamma events                         C
C ================================================================= C
                                                                    
C ================================================================= C
C You can use FORMOP to select approximate or exact formulae for    C
C generating gamma's. APPROXIMATE FORMULAE are faster and very well C
C suited only for the high energy part of the gamma spectrum        C
C (y ~ 1). Then, they are STRONGLY suggested (presently OBLIGED)    C
C for studying the PAIR BACKGROUND in the signal region (expecially C
C for radiative decay,since positron and photon energies and angles C 
C are CORRELATED). On the opposite, the use of the EXACT FORMULAE   C
C is STRONGLY suggested for studying the gamma background in wider  C
C ranges (for instance, for trigger studies).                       C
C ================================================================= C
C FORMOP = 0    : approximate formulae                              C
C FORMOP = 1    : exact formulae                                    C
C ================================================================= C

C ================================================================= C
C Finally, you can decide to generate events:                       C
C - over the whole solid angle (4*pi);                              C
C - in the selected range:                                          C
C         0.08<abs(cos(theta))<0.35 & 2*PI/3<phi<4*PI/3             C
C         for photons and the corresponding one for positrons;      C
C - in a fixed direction (only photons);                            C
C - in the extended angular range (only photons):                   C
C         abs(cos(theta))<0.35 & 11*PI/18<phi<25*PI/18              C
C - in a small solid angle around a fixed direction                 C
C This is done using the card IANG. The selected range option is    C 
C generally suggested in respect with the isotropic generation      C 
C since it is much less time consuming.                             C
C ================================================================= C
C IANG = 0     : whole solid angle                                  C
C IANG = 1     : selected range                                     C
C IANG = 2     : fixed direction                                    C
C IANG = 3     : extended range                                     C
C IANG = 4     : small range                                        C
C ================================================================= C

C ================================================================= C
C OUTPUTS:                                                          C
C ================================================================= C
C                                                                   C
C CODE(1) = GEANT code of 1st particle (2 = positron, 0 = nothing)  C
C CODE(2) = GEANT code of 2nd particle (1 = photon,   0 = nothing)  C
C                                                                   C
C PSTART(1,1:3) = positron starting momentum (in MeV !!!)           C
C PSTART(2,1:3) = photon   starting momentum (in MeV !!!)           C
C                                                                   C
C VSTART(1,1:3) = positron starting position                        C
C VSTART(2,1:3) = photon   starting position                        C
C ================================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      INTEGER I,J,ITYPE
      INTEGER CODE(2)
      REAL VSTART(2,3),PSTART(2,3)

      INTEGER ITIME
      DATA ITIME/0/

C Set and compute some preliminary parameters.
      IF (ITIME.EQ.0) THEN
         ITIME = 1
         CALL EVESET
      ENDIF

C Reset the kinematic vectors: VINIT = starting position, 
C                              PINIT = starting momentum.
C                              GCODE = GEANT particle code
      DO I = 1,2
         DO J = 1,3
            VINIT(I,J) = 0.
            PINIT(I,J) = 0.
         ENDDO
         GCODE(I) = 0
      ENDDO

C Generate events.
      IF      (ITYPE.EQ.SIGNAL) THEN
         CALL EVESIG
      ELSE IF (ITYPE.EQ.RADDEC) THEN
         CALL EVERDE
      ELSE IF (ITYPE.EQ.MICHRD) THEN
         CALL EVEACC(3)
      ELSE IF (ITYPE.EQ.MICHAF) THEN
         CALL EVEACC(4)
      ELSE IF (ITYPE.EQ.SIGNAE) THEN
         CALL EPOMEG
      ELSE IF (ITYPE.EQ.MICHEE) THEN
         CALL EPOMIC
      ELSE IF (ITYPE.EQ.NOISEE) THEN
         CALL EPONOI
      ELSE IF (ITYPE.EQ.SIGNAG) THEN
         CALL EPHMEG
      ELSE IF (ITYPE.EQ.RDGAMA) THEN
         CALL EPHRAD
      ELSE IF (ITYPE.EQ.AFGAMA) THEN
         CALL EPHANF
      ELSE IF (ITYPE.EQ.LFGAMA) THEN
         CALL EPHFLA
      ELSE IF (ITYPE.EQ.PILRAD) THEN
         CALL EPILRA
      ELSE IF (ITYPE.EQ.PILAIF) THEN
         CALL EPILAF
      ELSE IF (ITYPE.EQ.PILRAI) THEN
         CALL EPILRF
      ELSE IF (ITYPE.EQ.PILFLA) THEN
         CALL EPILFL
      ELSE
         WRITE(*,*) ' Invalid code for ITYPE (not defined) : ',ITYPE
      ENDIF

C Store GCODE,VINIT and PINIT in three other vectors for external 
C communication.
      CALL EVESTO(CODE,VSTART,PSTART)

      RETURN
      END

C ============================================================= C

      SUBROUTINE EVESTO(CODE,VSTART,PSTART)

C ============================================================= C
C Store GCODE,VINIT and PINIT in three other vectors for        C
C external communication.                                       C 
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      INTEGER I,J,CODE(2)
      REAL VSTART(2,3),PSTART(2,3)

      DO I = 1,2
         CODE(I) = GCODE(I)
         DO J = 1,3
            VSTART(I,J) = VINIT(I,J)
            PSTART(I,J) = PINIT(I,J)
         ENDDO
      ENDDO

      RETURN
      END

C ============================================================= C

      SUBROUTINE EVEPRI

C ============================================================= C
C Print the selected options.                                   C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Angular option.
      IF (IANG.EQ.0) THEN
         WRITE(*,*) ' Events generated in the full solid angle '
      ELSE IF (IANG.EQ.1) THEN
         WRITE(*,*) ' Events generated in the selected angular range '
      ELSE IF (IANG.EQ.2) THEN
         WRITE(*,*) ' Events generated in fixed direction '
         WRITE(*,*) ' Theta, phi = ',thgen,phgen
      ELSE IF (IANG.EQ.3) THEN
         WRITE(*,*) ' Events generated in extended angular range '
      ELSE IF (IANG.EQ.4) THEN
         WRITE(*,*) ' Events generated in a small angular range '
         WRITE(*,*) ' Delta Cos(theta) = ',DCTHET,' Delta PHI = ', DPHI
      ELSE
         WRITE(*,*) ' Invalid angular flag: IANG = ',IANG
         STOP' Abnormal End '
      ENDIF   

C Formulae option.
      IF (FORMOP.EQ.0) THEN
         WRITE(*,*) ' Approximate formulae used '
      ELSE IF (FORMOP.EQ.1) THEN
         WRITE(*,*) ' Exact formulae used '
      ELSE
         WRITE(*,*) ' Invalid choice of formulae: FORMOP = ',FORMOP
         STOP' Abnormal End '
      ENDIF
      
C Muon polarization.
      WRITE(*,*) ' Muon polarization = ',POL
      
C Maximum allowed differences of positron and gamma energies 
C from the kinematical bound (m_mu/2).
      WRITE(*,*) ' DX,DY = ',DX,DY

C Target.
      IF (ITAR.EQ.1) THEN
         WRITE(*,*) ' Extended target '
      ELSE IF (ITAR.EQ.0) THEN
         WRITE(*,*) ' Pointlike target '
      ELSE
         WRITE(*,*) ' Invalid target choice: ITAR = ',ITAR
         STOP' Abnormal End '
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE EVECAR

C ============================================================= C
C Read standard cards.                                          C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Read cards defining event properties.
      OPEN(5,FILE = 'careve.dat',STATUS = 'old')
      READ(5,*) TYPE
      READ(5,*) IANG
      READ(5,*) THGEN,PHGEN
      READ(5,*) DCTHET,DPHI
      READ(5,*) FORMOP
      READ(5,*) POL
      READ(5,*) DX
      READ(5,*) DY
      READ(5,*) ITAR
      CLOSE (5)

      RETURN
      END

C ================================================================= C

      SUBROUTINE EVESTA

C ================================================================= C
C Initialize some statistics about the event generation (i.e.       C
C number of rejected events).                                       C
C ================================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      NREJ = 0

      RETURN
      END

C ============================================================= C

      SUBROUTINE EVPRST

C ============================================================= C
C Print statistics about the event generation.                  C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      WRITE(*,*) ' Number of rejected events ',NREJ

      RETURN
      END

C ================================================================= C

      SUBROUTINE EVESIG

C ================================================================= C
C Generate SIGNAL EVENTS.                                           C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL PPE,PPG,EEE,EEG

C GEANT particle type: 1 = gamma, 2 = positron. The 
C first particle to be generated is the positron.
      GCODE(1) = 2
      GCODE(2) = 1

C Positron and photon momenta are fixed by the kinematics.
      PPE = 0.5*MUMMEV - 0.5*R*MUMMEV
      PPG = PPE

C Energies.
      EEG = PPG
      EEE = PPE + MUMMEV*R

C Extract the photon momentum.
      CALL PHOSGE(PPG)

C Extract the positron momentum.
      CALL POSSGE(PPE,1)

C Positron and photon position.
      CALL MEGVER(1)

C Relative angle.
      CALL SIGANG

      RETURN
      END

C ================================================================= C

      SUBROUTINE PHOSGE(PPG)

C ================================================================= C
C Generate MOMENTUM of PHOTON SIGNAL EVENTS.                        C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL PPG
      REAL ARNDM(2)
      REAL STHPH,PHIG,CTH
      INTEGER ICHK

C Extract angles.
C IANG = 0: angles generated isotropically 
C IANG = 1:    "       "     in the range of interest:
C                              0.08 < ABS(COS(THETA)) < 0.35 & 
C                            2*PI/3 <    PHI          < 4*PI/3
C IANG = 2:    "       "     in a fixed direction.
C IANG = 3:    "       "     in the extended range:
C                                     ABS(COS(THETA)) < 0.35 & 
C                           11*PI/18 <    PHI         < 25*PI/18
C IANG = 4:    "       "     in a small range around a fixed direction
 1    CONTINUE

      CALL GRNDM(ARNDM,2)

C IANG = 0: isotropical emission over 4*pi.
      IF (IANG.EQ.0) THEN
         CTHPH = 2.*ARNDM(1) - 1.
         PHIG = TWOPI*ARNDM(2)

C IANG = 1: emission in the selected angular range.
      ELSE IF (IANG.EQ.1) THEN
         CTHPH = 0.35*(2.*ARNDM(1) - 1.)
         PHIG = TWOPI/3.*(1.+ ARNDM(2))

C IANG = 2: emission in a fixed direction.
      ELSE IF (IANG.EQ.2) THEN
         CTHPH = COS(THGEN)
         PHIG = PHGEN

C IANG = 3: emission in the extended angular range.
      ELSE IF (IANG.EQ.3) THEN
         CTHPH = 0.35*(2.*ARNDM(1) - 1.)
         PHIG = PI/18.*(11. + 14.*ARNDM(2))

C IANG = 4: emission in small solid angle around a fixed direction.
      ELSE IF (IANG.EQ.4) THEN
         CTHPH = COS(THGEN) + (2.*ARNDM(1) - 1.)*DCTHET         
         PHIG = PHGEN + (2.*ARNDM(2) - 1.)*DPHI         

      ENDIF

C Reject useless events (only for IANG = 1).
      IF (IANG.EQ.1) THEN
         ICHK = 0
         CTH = CTHPH
         CALL CHKANG(CTH,ICHK)
         IF (ICHK.EQ.1) GOTO 1
      ENDIF

      STHPH = SQRT(1. - CTHPH*CTHPH)

C Photon momentum.
      PINIT(2,1) = PPG*STHPH*COS(PHIG)  
      PINIT(2,2) = PPG*STHPH*SIN(PHIG)  
      PINIT(2,3) = PPG*CTHPH

      RETURN
      END

C ============================================================= C

      SUBROUTINE CHKANG(CTH,ICHK)

C ============================================================= C
C Check if the angles were generated in the correct range.      C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      REAL CTH
      INTEGER ICHK
 
      ICHK = 0
      IF (ABS(CTH).LT.0.08) THEN
         NREJ = NREJ + 1
         IF (NREJ.LT.10.OR.(MOD(NREJ,100).EQ.0)) WRITE(*,*) 
     +' Rejected ',NREJ,' Events '
         ICHK = 1
      ENDIF

      RETURN
      END
      
C ============================================================= C

      SUBROUTINE POSSGE(PPE,GENCOD)

C ================================================================= C
C Generate MOMENTUM of POSITRON SIGNAL EVENTS.                      C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      INTEGER GENCOD
      REAL V(3),PPE
      REAL STHPOS,PHIPOS,CTH
      REAL ARNDM(2)
      INTEGER ICHK

C GENCOD = 1: pair event   => e+ momentum is fixed by gamma momentum
C GENCOD = 2: single event => e+ momentum must be generated 

      IF (GENCOD.EQ.1) THEN

C Positron momentum in the opposite direction to the photon.
         PINIT(1,1) = - PINIT(2,1)
         PINIT(1,2) = - PINIT(2,2)
         PINIT(1,3) = - PINIT(2,3)

C Positron emission angle.
         CTHPOS = PI - CTHPH

      ELSE IF (GENCOD.EQ.2) THEN

C Isotropic generation
         IF (IANG.EQ.0) THEN

C Positron direction cosines. 
            CALL RANVEC(V)

C Save the CTHPOS value for histos.
            CTHPOS = V(3)
         
C Positron momentum.
            PINIT(1,1) = PPE*V(1)
            PINIT(1,2) = PPE*V(2)
            PINIT(1,3) = PPE*V(3)

C Generation in the selected angular range or in the extended range.
         ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN

 1          CONTINUE
            CALL GRNDM(ARNDM,2)

            CTHPOS = -0.35*(2.*ARNDM(1) - 1.)
            IF (IANG.EQ.1) THEN
               PHIPOS = TWOPI/3.*(1.+ ARNDM(2)) + PI
            ELSE IF (IANG.EQ.3) THEN
               PHIPOS = PI/18.*(11. + 14.*ARNDM(2)) + PI
            ENDIF
            IF (PHIPOS.GT.TWOPI) PHIPOS = PHIPOS - TWOPI

            CTH = CTHPOS

            IF (IANG.EQ.1) THEN
               CALL CHKANG(CTH,ICHK)
               IF (ICHK.EQ.1) GOTO 1
            ENDIF

            STHPOS = SQRT(1. - CTHPOS*CTHPOS)

            PINIT(1,1) = PPE*STHPOS*COS(PHIPOS)
            PINIT(1,2) = PPE*STHPOS*SIN(PHIPOS)
            PINIT(1,3) = PPE*CTHPOS
         ENDIF

      ENDIF

      RETURN
      END

C ================================================================= C

      SUBROUTINE SIGANG

C ================================================================= C
C RELATIVE ANGLE between positron and photon in SIGNAL EVENTS. It   C
C MUST BE 180 DEGREES; then, this routine is a check tool for the   C
C reliability of the generation.                                    C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL VDOT
      REAL VPO(3),VPH(3)
      REAL SCAPR,SCAMO
      INTEGER I

      DO I = 1,3
         VPO(I) = PINIT(1,I)
         VPH(I) = PINIT(2,I)
      ENDDO

      SCAPR = VDOT(VPO,VPH,3)
      SCAMO = VDOT(VPO,VPO,3)*VDOT(VPH,VPH,3)
      SCAMO = SQRT(SCAMO)

      Z = ACOS(SCAPR/SCAMO)

      IF (ABS(Z - PI).GT.0.001) THEN
         WRITE(*,*) ' Error in signal generation: Z = ',Z
         STOP' Abnormal End '
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE RANVEC(V)

C ============================================================= C
C Generate a 3-dimensional random unitary vector.               C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL V(3),FI,SENTETA,CTH
      REAL ARNDM(2)
      INTEGER ICHK

 1    CONTINUE
      CALL GRNDM(ARNDM,2)
 
C Isotropical emission over 4*pi.
      V(3) = 2.*ARNDM(1) - 1.
      FI = TWOPI*ARNDM(2)

C Direction cosines in the (X,Y) plane.
      SENTETA = SQRT(1. - V(3)*V(3))
      V(1) = SENTETA*COS(FI)
      V(2) = SENTETA*SIN(FI)

      RETURN
      END

C ================================================================== C

      SUBROUTINE EVESET

C ================================================================== C
C Computes the ratio between electron and muon masses and convert    C
C the muon and electron mass from GeV (as given by GEANT) to MeV.    C
C ================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      R = (EMASS/EMMU)*(EMASS/EMMU)
      EMMEV  = 1.E+3*EMASS
      MUMMEV = 1.E+3*EMMU
      EMAX = 0.5*MUMMEV
      PMAX = EMAX - 0.5*EMMEV*EMMEV/EMAX

      RETURN
      END

C ================================================================== C

      SUBROUTINE EVERDE

C ================================================================== C
C It returns the particle codes, starting positions and momenta for  C
C POSITRON and GAMMA's emitted in RADIATIVE MUON DECAY:              C
C     		     mu -> e+ nu antinu gamma                        C
C ================================================================== C

      IMPLICIT NONE
 
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"
     
      REAL EPH,EPOS,PPOS,PHDIR(3),POSDIR(3)
      INTEGER I

C GEANT Code: 2 = POSITRON, 1 = GAMMA.
      GCODE(1) = 2
      GCODE(2) = 1

C Positron direction cosines.
      CALL ERDPOA(POSDIR)          
     
C Energy distribution of outgoing positron and photon.
      CALL ERDPOE(EPOS,EPH)

C Value of positron momentum.
      PPOS = SQRT(EPOS*EPOS - EMMEV*EMMEV)

C Photon direction cosines.
      CALL ERDPHA(0,PHDIR)

C Initial position of positron and photon.
      CALL MEGVER(1)

C Positron and photon momenta.
      DO 1 I = 1,3
         PINIT(1,I) = POSDIR(I)*PPOS
         PINIT(2,I) = PHDIR(I)*EPH
 1    CONTINUE
     
      RETURN
      END

C ================================================================== C

      SUBROUTINE ERDPOA(PDIR)

C ================================================================== C
C Generate MOMENTUM of outgoing POSITRONS in RADIATIVE DECAY MODE.   C 
C The angle is measured from the MUON SPIN DIRECTION.                C 
C ================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      INTEGER ITIME
      DATA ITIME/0/
     
      INTEGER ICHK
      REAL ARNDM,YYY,PDIR(3),PHIPOS,STHPOS,CTH

      REAL ERDPAN
      EXTERNAL ERDPAN
 
      REAL HRNDM1
      LOGICAL HEXIST

      IF (ITIME.EQ.0) THEN
         ITIME = 1
         IF (FORMOP.EQ.0) THEN
            IF (.NOT.HEXIST(670))  
     +CALL HBFUN1(670,'Positron angular distribution',
     +200,-1.,1.,ERDPAN)
         ELSE IF (FORMOP.EQ.1) THEN
            WRITE(*,*) ' Only approximate formulae are allowed for '
            WRITE(*,*) ' correlated events: change FORMOP to 0 !'
            STOP' Abnormal End '
         ENDIF
      ENDIF

 1    CONTINUE

C IANG = 0: isotropic emission over 4*pi.
      IF (IANG.EQ.0) THEN

         CTHPOS = -0.35*HRNDM1(670)
         CALL GRNDM(ARNDM,1)
         PHIPOS = TWOPI*ARNDM

C IANG = 1: isotropic generation in the selected angular range.
      ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN

         CTHPOS = -0.35*HRNDM1(670)
         CALL GRNDM(ARNDM,1)
         IF (IANG.EQ.1) THEN
            PHIPOS = TWOPI/3.*(1.+ ARNDM) + PI
         ELSE IF (IANG.EQ.3) THEN
            PHIPOS = PI/18.*(11. + 14.*ARNDM) + PI
         ENDIF
         IF (PHIPOS.GT.TWOPI) PHIPOS = PHIPOS - TWOPI
      ENDIF

C Reject useless events (only for IANG = 1).
      IF (IANG.EQ.1) THEN
         ICHK = 0
         CTH = CTHPOS
         CALL CHKANG(CTH,ICHK)
         IF (ICHK.EQ.1) GOTO 1
      ENDIF

      STHPOS = SQRT(1. - CTHPOS*CTHPOS)

C Positron direction cosines.
      PDIR(1) = STHPOS*COS(PHIPOS)
      PDIR(2) = STHPOS*SIN(PHIPOS)
      PDIR(3) = CTHPOS

      RETURN
      END

C ================================================================== C

      REAL FUNCTION ERDPAN(CTH)

C ================================================================== C
C ANGULAR DISTRIBUTION of positrons in respect to the MUON SPIN      C
C DIRECTION in RADIATIVE DECAY MODE; it becomes ISOTROPIC when       C
C POL = 0.                                                           C
C Reference: Kuno & Okada, PHYS. REV. LETT. 77,(1996),435            C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc" 
   
      REAL CTH
      REAL J1,J2

      INTEGER ITIME
      DATA ITIME/0/

      IF (ITIME.EQ.0) THEN
         J1 =       (DX**4)*(DY**2)
         J2 = 8./3.*(DX**3)*(DY**3)
         ITIME = 1
      ENDIF

      IF (POL.NE.0.) THEN
        ERDPAN = J1*(1.- POL*CTH) + J2*(1.+ POL*CTH)
      ELSE
        ERDPAN = 1.
      ENDIF

      RETURN
      END

C ================================================================== C

      SUBROUTINE ERDPOE(EPO,EGAMMA)

C ================================================================== C
C ENERGY of outgoing POSITRON and PHOTON in RADIATIVE DECAY MODE.    C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C ================================================================== C
C DX and DY are the maximum allowed differences of the POSITRON      C 
C AND PHOTON ENERGY from the upper kinematical bound (EMMEV/2).      C
C DZ is the maximum difference between the POSITRON - PHOTON         C 
C ANGLE and PI. When the positron and photon energies are            C 
C fixed, DZ IS CONSTRAINED BY THE KINEMATICS to be within the range  C 
C          0 < DZ < SQRT(2*(1 - DX)*(1 - DY))                        C
C ================================================================== C

      REAL EPO,EGAMMA

      INTEGER ITIME
      DATA ITIME/0/

C ================================================================== C
C ERDDIS is the POSITRON & PHOTON energy distribution function.      C
C Reference Kuno & Okada, PHYS. REV. LETT. 77,(1996),434             C
C ================================================================== C

      REAL ERDDIS,XX,YY
      EXTERNAL ERDDIS

      IF (ITIME.EQ.0.OR.(ITIME.EQ.1.AND.POL.NE.0)) THEN
         ITIME = 1
         IF (FORMOP.EQ.0) THEN
            CALL HBFUN2(1234,'Positron and Photon Energy distribution',
     +                   100,(1.- DX),1.,100,(1.- DY),1.,ERDDIS)
         ELSE IF (FORMOP.EQ.1) THEN
            WRITE(*,*) ' Only approximate formulae are allowed for '
            WRITE(*,*) ' correlated events: change FORMOP to 0 !'
            STOP' Abnormal End '
         ENDIF
      ENDIF

      CALL HRNDM2(1234,XX,YY)

C ================================================================== C
C Only for polarized muons.                                          C 
C ================================================================== C
      IF (POL.NE.0) CALL HDELET(1234)

C Positron and photon energies.
      EPO = EMAX*XX
      EGAMMA = EMAX*YY

C To generate the relative angle.
      X = XX
      Y = YY

      RETURN
      END

C ================================================================== C

      REAL FUNCTION ERDDIS(XX,YY)

C ================================================================== C
C ENERGY DISTRIBUTION of outgoing POSITRON and PHOTON in RADIATIVE   C
C DECAY MODE.                                                        C
C Reference Kuno & Okada, PHYS. REV. LETT. 77,(1996),434             C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      REAL XX,YY
      
      IF (POL.NE.0.) THEN
         ERDDIS = (2.*(1.- XX)**3*(1.- YY)*(1.- POL*CTHPOS) + 
     +	           6.*(1.- XX)**2*(1.- YY)**2*(1.+ POL*CTHPOS))
      ELSE
         ERDDIS = (2.*(1.- XX)**3*(1.- YY) + 
     +             6.*(1.- XX)**2*(1.- YY)**2)
      ENDIF

      RETURN
      END

C ================================================================== C

      SUBROUTINE ERDPHA(IPIL,PHDIR)

C ================================================================== C
C ANGULAR DISTRIBUTION of outgoing PHOTON in RADIATIVE DECAY MODE.   C
C IPIL = 0: usual case;                                              C
C IPIL = 1: pile-up photons; in this case a further check is needed  C
C           to have the photon in the correct angular range.         C
C ================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL CMIN,THPH
      REAL ARNDM
      REAL PHDIR(3),PHI,STHPH
      INTEGER IPIL

      DZ = SQRT(2.*(1.- X)*(1.- Y))      
      CMIN = COS(DZ)

C ==================================================================== C
C The angular distribution is assumed FLAT within the kinematical      C 
C bounds. THIS IS NOT CORRECT, but THE ERROR SHOULD BE VERY SMALL      C
C since for DX = 0.1 and DY = 0.1 (both large) THE MAXIMUM DZ VALUE IS:C  
C      (DZ)_MAX = SQRT(0.02) = 0.14 = 8.1 DEGREES.                     C       
C The average value should be about 3 DEGREES.                         C
C ==================================================================== C

      IF (IPIL.EQ.0.OR.(IPIL.EQ.1.AND.(IANG.NE.1.AND.IANG.NE.3))) THEN
         IF (FORMOP.EQ.0) THEN
            CALL GRNDM(ARNDM,1)
            Z = ACOS(1. - ARNDM*(1. - CMIN))
         ELSE IF (FORMOP.EQ.1) THEN
            WRITE(*,*) ' Only approximate formulae are allowed for '
            WRITE(*,*) ' correlated events: change FORMOP to 0 !'
            STOP' Abnormal End '
         ENDIF

C Relative angle between positron and photon.
         Z = PI - Z

C This condition ensures that the photon is assumed backward 
C in respect with the positron which is correctly generated.
      ELSE IF (IPIL.EQ.1.AND.(IANG.EQ.1.OR.IANG.EQ.3)) THEN
         Z = PI
      ENDIF

C Photon angle.
      THPH = Z + ACOS(CTHPOS)
      CTHPH = COS(THPH) 
      STHPH = SQRT(1. - CTHPH*CTHPH)

      CALL GRNDM(ARNDM,1)

      IF (IPIL.EQ.0.OR.(IPIL.EQ.1.AND.(IANG.NE.1.AND.IANG.NE.3))) THEN
         PHI = TWOPI*ARNDM
      ELSE IF (IPIL.EQ.1.AND.(IANG.EQ.1.OR.IANG.EQ.3)) THEN
         PHI = TWOPI/3.*(1.+ ARNDM)
      ENDIF

C Azimuthal symmetry is assumed as usually.
      PHDIR(1) = STHPH*COS(PHI)
      PHDIR(2) = STHPH*SIN(PHI)
      PHDIR(3) = CTHPH

      RETURN
      END

C ================================================================== C

      SUBROUTINE EVEACC(IPRO)

C ================================================================== C
C It returns starting POSITIONS and MOMENTA for                      C 
C POSITRON and GAMMA's involved in ACCIDENTAL PROCESSES.             C
C ================================================================== C

C ================================================================== C
C          IPRO = 3: ACCIDENTAL COINCIDENCE with:                    C
C                    e+    from USUAL DECAY,                         C
C                    gamma from RADIATIVE DECAY                      C
C          IPRO = 4: ACCIDENTAL COINCIDENCE with:                    C
C                    e+    from USUAL DECAY,                         C
C                    gamma from e+ ANNIHILATION IN FLIGHT            C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      INTEGER IPRO,I
 
      REAL PPH(3),EPH,PPOS(3),EPOS

      GCODE(1) = 2
      GCODE(2) = 1

C Positron energy and momentum.
      CALL EACPOE

C Photon energy and emission angle.
      CALL EACPHE(IPRO)

C Relative angle between positron and photon.
      CALL EACRLA

C Initial position of positron and photon.
      CALL MEGVER(4)

      RETURN
      END

C ================================================================== C

      SUBROUTINE EACPOE

C ================================================================== C
C ENERGY and ANGULAR DISTRIBUTION of outgoing electron in            C
C ACCIDENTAL PROCESSES (MICHEL spectrum).                            C
C ================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL STHPOS,PHIPOS,CTH
      INTEGER ICHK

      REAL MICHEL
      EXTERNAL MICHEL

      REAL MICANG
      EXTERNAL MICANG

      REAL ARNDM,HRNDM1

      REAL EPOS,POSMOM
  	
      INTEGER ITIME
      DATA ITIME/0/

      LOGICAL HEXIST

C ================================================================== C
C ANGULAR DISTRIBUTION: if the muon is UNPOLARIZED, it is ISOTROPIC; C
C otherwise, it has the form (A + B*COS(THETA)); B = POL/3.          C
C ================================================================== C

      IF (ITIME.EQ.0) THEN
        ITIME = 1
        IF (.NOT.HEXIST(668))  
     +CALL HBFUN1(668,' Michel positrons angular distribution ',
     +200,-1.,1.,MICANG)

C Positron energy spectrum.
        IF (POL.EQ.0.) THEN
           IF (.NOT.HEXIST(667))  
     +      CALL HBFUN1(667,'Michel spectrum',200,(1.- DX),1.,MICHEL)
        ENDIF
      ENDIF

 1    CONTINUE

C IANG = 0 : generation over 4*pi.
      IF (IANG.EQ.0) THEN
         CTHPOS = HRNDM1(668)
         CALL GRNDM(ARNDM,1)
         PHIPOS = TWOPI*ARNDM

C IANG = 1 : generation in the selected range. 
      ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN
         CTHPOS = -0.35*HRNDM1(668) 
         CALL GRNDM(ARNDM,1)
         IF (IANG.EQ.1) THEN
            PHIPOS = TWOPI/3.*(1.+ ARNDM) + PI
         ELSE IF (IANG.EQ.3) THEN
            PHIPOS = PI/18.*(11. + 14.*ARNDM) + PI
         ENDIF
         IF (PHIPOS.GT.TWOPI) PHIPOS = PHIPOS - TWOPI
      ENDIF

C Reject useless events (only for IANG = 1).
      IF (IANG.EQ.1) THEN
         ICHK = 0
         CTH = CTHPOS
         CALL CHKANG(CTH,ICHK)
         IF (ICHK.EQ.1) GOTO 1
      ENDIF

      STHPOS = SQRT(1.- CTHPOS*CTHPOS)

C ================================================================== C
C Positron energy spectrum. The MICHEL spectrum for UNPOLARIZED      C 
C MUONS CAN BE COMPUTED ONCE, but for POLARIZED MUONS MUST BE        C 
C COMPUTED EVERY TIME since it depends ON POL*CTHETA.                C
C ================================================================== C
      IF (POL.NE.0.) 
     +   CALL HBFUN1(667,'Michel spectrum',200,(1.- DX),1.,MICHEL)
      EPOS = EMAX*HRNDM1(667)      
      IF (POL.NE.0.) CALL HDELET(667)

C Positron momentum.
      POSMOM = SQRT(EPOS*EPOS - EMMEV*EMMEV)

C Positron momentum components.
      PINIT(1,1) = POSMOM*STHPOS*COS(PHIPOS)
      PINIT(1,2) = POSMOM*STHPOS*SIN(PHIPOS)
      PINIT(1,3) = POSMOM*CTHPOS

      RETURN
      END

C ================================================================== C

      REAL FUNCTION MICANG(CTHETA)

C ================================================================== C
C ANGULAR DISTRIBUTION of outgoing POSITRONS in STANDARD MUON DECAY  C 
C (MICHEL process). The positron angle is computed in respect to the C 
C MUON SPIN DIRECTION.                                               C
C Reference: L.B. Okun, "LEPTONS AND QUARKS", CHAPTER 3              C
C ================================================================== C

      IMPLICIT NONE
      REAL CTHETA

#include"uniinc/megeve.inc"
     
C For unpolarized muons the distribution becomes isotropic.
      IF (POL.NE.0.) THEN
        MICANG = 1. + POL/3.*CTHETA
      ELSE
        MICANG = 1.
      ENDIF

      RETURN
      END

C ================================================================== C

      REAL FUNCTION MICHEL(EPSI)

C ================================================================== C
C MICHEL spectrum of electrons emitted in USUAL MUON DECAY.          C
C The spectrum is normalized for use in conjunction with other       C
C functions to generate the annihilation photon spectrum.            C
C Reference: L.B. Okun, "LEPTONS AND QUARKS", CHAPTER 3              C
C ================================================================== C

      IMPLICIT NONE
      REAL EPSI

#include"uniinc/megeve.inc"
     
C Polarized muons.
      IF (POL.NE.0.) THEN
        MICHEL = 2.*((3. - 2.*EPSI) - 
     +               (1. - 2.*EPSI)*POL*CTHPOS)*EPSI*EPSI
      ELSE

C Unpolarized muons.
        MICHEL = 2.*(3. - 2.*EPSI)*EPSI*EPSI
      ENDIF

      RETURN
      END

C ================================================================== C

      SUBROUTINE EACPHE(IPRO)

C ================================================================== C
C ENERGY and MOMENTUM of outgoing photon in ACCIDENTAL PROCESSES.    C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      INTEGER IPRO,I

C IPRO = 3: Radiative decay.
C IPRO = 4: Annihilation in flight.

C Radiative decay
      IF (IPRO.EQ.3) THEN
        CALL EPHDEC

C Annihilation in flight.
      ELSE IF (IPRO.EQ.4) THEN
        CALL EPHANN(0)
      ENDIF
     
      RETURN
      END

C ================================================================== C

      SUBROUTINE EPHDEC

C ================================================================== C
C ENERGY and MOMENTUM of photons emitted in RADIATIVE DECAY. The     C 
C photon angle is measured from the muon spin direction.             C
C ================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL STHPH,PHIPH
      REAL EPH,E,CT,CTH
      REAL EEE,BBB,ARNDM,HRNDM1
      INTEGER ICHK

      REAL EDEDIS,RADEC
      EXTERNAL EDEDIS,RADEC

      INTEGER ITIME
      DATA ITIME/0/

C If the muon is UNPOLARIZED, the ENERGY and ANGULAR DISTRIBUTION 
C can be COMPUTED ONCE, but, if the muon is POLARIZED, they MUST BE 
C COMPUTED EVERY TIME.

      IF (ITIME.EQ.0) THEN
        ITIME = 1

C FORMOP = 1 is suggested for gamma only events. In this case the 
C photon angle is generated isotropically (in the selected range, 
C if requested). 
        IF (FORMOP.EQ.0) THEN
          IF (POL.EQ.0.) THEN
            IF (IANG.EQ.0) THEN
              CALL HBFUN2(1235,'Photon energy and angular distribution',
     +                 200,(1.- DY),(1.- R),200,-1.,1.,EDEDIS)
            ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN
              CALL HBFUN2(1235,'Photon energy and angular distribution',
     +                 200,(1.- DY),(1.- R),70,-.35,.35,EDEDIS)
            ENDIF
          ENDIF
        ELSE IF (FORMOP.EQ.1) THEN
          CALL HBFUN1(671,' Photon energy distribution ',
     +                 200,(1.- DY),1.,RADEC)      
        ENDIF
      ENDIF

C Only for polarized muons.
      IF (FORMOP.EQ.0) THEN
         IF (POL.NE.0.) THEN
           IF (IANG.EQ.0) THEN
             CALL HBFUN2(1235,' Photon energy and angular distribution',
     +              200,(1.- DY),(1.- R),200,-1.,1.,EDEDIS)
           ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN
             CALL HBFUN2(1235,' Photon energy and angular distribution',
     +              200,(1.- DY),(1.- R),70,-.35,.35,EDEDIS)
           ENDIF
         ENDIF
         CALL HRNDM2(1235,E,CT)
      ELSE IF (FORMOP.EQ.1) THEN
         E = HRNDM1(671)   
      ENDIF

      EPH = E*EMAX
         
C Only for polarized muons.
      IF (FORMOP.EQ.0.AND.POL.NE.0.) CALL HDELET(1235)

 1    CONTINUE

      IF (FORMOP.EQ.0) THEN 
         CTHPH = CT

C Azimuthal symmetry is assumed as usual.
         IF (IANG.EQ.0) THEN
            CALL GRNDM(ARNDM,1)
            PHIPH = TWOPI*ARNDM
         ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN
            CALL GRNDM(ARNDM,1)
            IF (IANG.EQ.1) THEN
               PHIPH = TWOPI/3.*(1.+ ARNDM)
            ELSE IF (IANG.EQ.3) THEN
               PHIPH = PI/18.*(11. + 14.*ARNDM)
            ENDIF
         ENDIF

      ELSE IF (FORMOP.EQ.1) THEN

C IANG = 0: isotropical emission over 4*pi.
         IF (IANG.EQ.0) THEN
            CALL GRNDM(ARNDM,1)            
            CTHPH = 2.*ARNDM - 1.
            CALL GRNDM(ARNDM,1)
            PHIPH = TWOPI*ARNDM

C IANG = 1: isotropical emission in the selected range.
         ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN
            CALL GRNDM(ARNDM,1)
            CTHPH = 0.35*(2.*ARNDM - 1.)
            CALL GRNDM(ARNDM,1)
            IF (IANG.EQ.1) THEN
               PHIPH = TWOPI/3.*(1.+ ARNDM)
            ELSE IF (IANG.EQ.3) THEN
               PHIPH = PI/18.*(11. + 14.*ARNDM)
            ENDIF
         ENDIF

C Reject useless events (only for IANG = 1).
         IF (IANG.EQ.1) THEN
            ICHK = 0
            CTH = CTHPH
            CALL CHKANG(CTH,ICHK)
            IF (ICHK.EQ.1) GOTO 1
         ENDIF
      ENDIF

      STHPH = SQRT(1. - CTHPH*CTHPH)

C Photon momentum.
      PINIT(2,1) = EPH*STHPH*COS(PHIPH)
      PINIT(2,2) = EPH*STHPH*SIN(PHIPH)
      PINIT(2,3) = EPH*CTHPH

      RETURN
      END

C ================================================================== C

      REAL FUNCTION EDEDIS(YY,CTH)

C ================================================================== C
C ENERGY and ANGULAR DISTRIBUTION of photons in RADIATIVE DECAY.     C
C The angle is measured from the MUON SPIN DIRECTION.                C
C Reference: Kuno, Maki and Okada, PHYS. REV. D55, R2517, (1997)     C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      REAL YY,CTH

      REAL JPLUS,JMINUS
      REAL JP,JM

      JP = JPLUS(YY)
      JM = JMINUS(YY)

C Polarized muons.
      IF (POL.NE.0.) THEN
        EDEDIS = (JP*(1. + POL*CTH) + JM*(1. - POL*CTH))/YY
      ELSE   

C Unpolarized muons.
        EDEDIS = (JP + JM)/YY
      ENDIF

      RETURN
      END

C ================================================================== C

      REAL FUNCTION JPLUS(YY)

C ================================================================== C
C JPLUS,JMINUS: ENERGY DEPENDENT COEFFICIENTS of the photon          C
C ANGULAR DISTRIBUTION in RADIATIVE DECAY.                           C
C Reference: Kuno, Maki and Okada, PHYS. REV. D55, R2517, (1997)     C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      REAL YY
   
C Protection.
      IF (YY.EQ.0) THEN
        JPLUS = 0.
        RETURN
      ENDIF

      JPLUS = (1.- YY)*(
     + (  3.*ALOG((1.- YY)/R) - 8.5) + 
     + (- 3.*ALOG((1.- YY)/R) + 7.)*(1. - YY) + 
     + (  2.*ALOG((1.- YY)/R) - 13./3.)*(1.- YY)*(1.- YY)
     + )

      RETURN
      END

C ================================================================== C

      REAL FUNCTION JMINUS(YY)

C ================================================================== C
C JPLUS,JMINUS: ENERGY DEPENDENT COEFFICIENTS of the photon          C
C ANGULAR DISTRIBUTION in RADIATIVE DECAY.                           C
C Reference: Kuno, Maki and Okada, PHYS. REV. D55, R2517, (1997)     C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      REAL YY

C Protection.
      IF (YY.EQ.0) THEN
        JMINUS = 0.
        RETURN
      ENDIF

      JMINUS = (1.- YY)*(1.- YY)*(
     + (  3.*ALOG((1.- YY)/R) - 93./12.) + 
     + (- 4.*ALOG((1.- YY)/R) + 29./3.)*(1.- YY) + 
     + (  2.*ALOG((1.- YY)/R) - 55./12.)*(1.- YY)*(1.- YY)
     + )

      RETURN
      END

C ================================================================== C

      SUBROUTINE EPHANN(IPIL)

C ================================================================== C
C ENERGY and momentum of photons emitted in POSITRON - ELECTRON      C
C ANNIHILATION IN FLIGHT.                                            C
C Reference Y. S. Tsai, REV. MOD. PHYS. 46,815,(1974)                C
C IPIL = 0: usual case (no pile-up photons)                          C
C IPIL = 1: pile-up photons; in this case we need to check whether   C
C           the photon is emitted within the requested angular range C
C ================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL EVEANN,MICHEL,MICANG,FLANNI
      EXTERNAL EVEANN,MICHEL,MICANG,FLANNI
      REAL HRNDM1
      INTEGER IPIL

      REAL STHPH,ARNDM,SQG,CTG,CTH
      REAL PHIPH,CPHIPH,SPHIPH,PHIPOS,STHPOS
      REAL EPH
      INTEGER ICHK
      
      REAL CIN(3),COUT(3)

      INTEGER ITIME
      DATA ITIME/0/

      LOGICAL HEXIST

      IF (ITIME.EQ.0) THEN
         ITIME = 1

C FORMOP = 0 : approximate formulae.
         IF (FORMOP.EQ.0) THEN
            IF (.NOT.HEXIST(668))  
     +       CALL HBFUN1(668,' Michel positrons angular distribution ',
     +                         200,-1.,1.,MICANG)
            IF (POL.EQ.0.) THEN
               IF (.NOT.HEXIST(667)) 
     +             CALL HBFUN1(667,' Michel spectrum ',
     +                               200,(1.- DX),1.,MICHEL)
            ENDIF

C FORMOP = 1 : exact formulae.
         ELSE IF (FORMOP.EQ.1) THEN 
            CALL HBFUN1(669,'Annihilation photons',
     +                       200,(1.- DY),1.,FLANNI)
         ENDIF
      ENDIF

C ================================================================== C
C First, it extracts the positron angle in the detector frame.       C
C ================================================================== C

 1    CONTINUE

C IANG = 0: emission in 4*pi.
      IF (IANG.EQ.0) THEN
        CTHPOS = HRNDM1(668)
        CALL GRNDM(ARNDM,1)
        PHIPOS = TWOPI*ARNDM

C IANG = 1: emission in the selected range.
      ELSE IF (IANG.EQ.1.OR.IANG.EQ.3) THEN
        CTHPOS = -0.35*HRNDM1(668)
        CALL GRNDM(ARNDM,1)
        IF (IANG.EQ.1) THEN
           PHIPOS = TWOPI/3.*(1.+ ARNDM) + PI
        ELSE IF (IANG.EQ.3) THEN
           PHIPOS = PI/18.*(11. + 14.*ARNDM) + PI
        ENDIF
        IF (PHIPOS.GT.TWOPI) PHIPOS = PHIPOS - TWOPI
      ENDIF

C Reject useless events (only for IANG = 1).
      IF (IANG.EQ.1) THEN
         ICHK = 0
         CTH = CTHPOS
         CALL CHKANG(CTH,ICHK)
         IF (ICHK.EQ.1) GOTO 1
      ENDIF

      STHPOS = SQRT(1. - CTHPOS*CTHPOS)

C FORMOP = 0 : approximate formulae. 
C If FORMOP = 0 we need the positron energy which must be generated.
      IF (FORMOP.EQ.0) THEN
         IF (POL.NE.0.) 
     +     CALL HBFUN1(667,' Michel spectrum ',200,(1.- DX),1.,MICHEL)
         EPP = EMAX*HRNDM1(667)      
         IF (POL.NE.0.) CALL HDELET(667)
         PPP = SQRT(EPP*EPP - EMMEV*EMMEV)

C ================================================================== C 
C Now it extracts the photon energy.                                 C
C ================================================================== C

         CALL HBFUN1(669,'Annihilation photons',200,(1.- DY),1.,EVEANN)
         EPH = HRNDM1(669)*EPP
         CALL HDELET(669)

C FORMOP = 1 : exact formulae.
      ELSE IF (FORMOP.EQ.1) THEN
         EPH = HRNDM1(669)*EMAX
      ENDIF

C ================================================================== C
C Photon ANGULAR DISTRIBUTION. The photon direction is fixed         C
C at a fixed photon energy since the process is a                    C
C TWO-BODY -> TWO-BODY REACTION                                      C
C ================================================================== C
C N.B. THPH is measured from the POSITRON LINE OF FLIGHT.            C
C      THIS IS DIFFERENT FROM ALL THE PREVIOUS CASES,                C
C      when the reference direction was given by the MUON            C
C      SPIN DIRECTION !                                              C
C ================================================================== C

C Azimuthal symmetry is assumed as usual.
      CALL GRNDM(ARNDM,1)
      PHIPH  = TWOPI*ARNDM
      CPHIPH = COS(PHIPH)
      SPHIPH = SIN(PHIPH)

C FORMOP = 0 : positron energy varies from event to event.
      IF (FORMOP.EQ.0) THEN
        CTHPH = (EPP + EMMEV)*(EPH - EMMEV)/(EPH*PPP)

C FORMOP = 1 : we have already integrated over positron energy. 
      ELSE IF (FORMOP.EQ.1) THEN
        CTHPH = (EMAX + EMMEV)*(EPH - EMMEV)/(EPH*PMAX)
      ENDIF

      STHPH = SQRT(1. - CTHPH*CTHPH)

C Compute the positron direction cosines.
      CIN(1) = STHPOS*COS(PHIPOS)
      CIN(2) = STHPOS*SIN(PHIPOS)
      CIN(3) = CTHPOS

C Protection against dangerous angles.
      IF (CIN(3).GT.(+ 0.999)) CIN(3) = + 0.999
      IF (CIN(3).LT.(- 0.999)) CIN(3) = - 0.999

      IF (IPIL.EQ.0.OR.(IPIL.EQ.1.AND.IANG.NE.1.AND.IANG.NE.3)) THEN

C Transformation.
         SQG = SQRT(1. - CIN(3)*CIN(3))
         COUT(3) = CIN(3)*CTHPH + STHPH*CPHIPH*SQG
         CTG = CTHPH - COUT(3)*CIN(3)
         COUT(2) = (CIN(2)*CTG + STHPH*SPHIPH*CIN(1)*SQG)/(SQG*SQG)
         COUT(1) = (CIN(1)*CTG - STHPH*SPHIPH*CIN(2)*SQG)/(SQG*SQG)

      ELSE IF (IPIL.EQ.1.AND.(IANG.EQ.1.OR.IANG.EQ.3)) THEN

         COUT(1) = - CIN(1)
         COUT(2) = - CIN(2)
         COUT(3) = - CIN(3)
      ENDIF

C Photon momentum.
      PINIT(2,1) = EPH*COUT(1)
      PINIT(2,2) = EPH*COUT(2)
      PINIT(2,3) = EPH*COUT(3)

      RETURN
      END

C ================================================================== C
     
      REAL FUNCTION EVEANN(YY)

C ================================================================== C
C ENERGY DISTRIBUTION of photons from POSITRON - ELECTRON            C
C ANNIHILATION IN FLIGHT.                                            C
C Reference Y. S. Tsai, REV. MOD. PHYS. 46,815,(1974)                C
C ================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL YY,YYY	

C Protection.
      IF (YY.EQ.0.) THEN
        EVEANN = 0.
        RETURN
      ENDIF

      GAMMA = EPP/EMMEV
      YYY = YY/(2.*GAMMA) + (1.- YY)
      EVEANN = YY/YYY + YYY/YY

      RETURN
      END

C ================================================================== C

      SUBROUTINE EACRLA

C ================================================================== C
C RELATIVE ANGLE between positron & photon in ACCIDENTAL PROCESSES.  C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"
     
      REAL TPH

C ================================================================== C
C Remember that in case of PHOTONS emitted from RADIATIVE MUON DECAY,C 
C the angle is measured in respect to THE MUON SPIN DIRECTION, while C 
C in case of PHOTONS emitted from POSITRON ANNIHILATION, it is       C
C measured from the POSITRON LINE OF FLIGHT.                         C
C ================================================================== C

      TPH = ACOS(CTHPH)
      Z = TPH 
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPOMEG

C ============================================================= C
C POSITRONS from MU -> E+ GAMMA decay (52.8 MeV).               C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL PPE,EEE

C The positron momentum is fixed by the kinematics (Two-body decay). 
      PPE = 0.5*MUMMEV - 0.5*R*MUMMEV

C Positron energy.
      EEE = PPE + MUMMEV*R

C Positron momentum.
      CALL POSSGE(PPE,2)

C Positron position.
      CALL MEGVER(2)

C Putting GCODE(2) = 0 the photon is excluded.
      GCODE(1) = 2
      GCODE(2) = 0
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPOMIC

C ============================================================= C
C POSITRONS from MU -> E+ NU NUBAR decay (Michel).              C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Positron energy and momentum. 
      CALL EACPOE

C Positron position.
      CALL MEGVER(2)

C Putting GCODE(2) = 0 the photon is excluded.
      GCODE(1) = 2
      GCODE(2) = 0

      RETURN
      END

C ============================================================= C

      SUBROUTINE EPONOI

C ============================================================= C
C POSITRON noise on spectrometer.                               C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Positron momentum: the positron is assumed nearly at rest (0.25 MeV).
      PINIT(1,1) = 0.25
      PINIT(1,2) = 0.25
      PINIT(1,3) = 0.25

C Positron position. We check that the positron position is 
C on chambers. 
      CALL MEGVER(5)

C Putting GCODE(2) = 0 the photon is excluded.
C For this particular case we put GCODE(1) = 3 (not 2!) to 
C avoid positron annihilation => spurious photons etc. 
C We are really generating ELECTRONS, NOT POSITRONS in this routine.
      GCODE(1) = 3
      GCODE(2) = 0

      RETURN
      END

C ============================================================= C

      SUBROUTINE EPHMEG

C ============================================================= C
C PHOTONS from MU -> E+ GAMMA decay (52.8 MeV).                 C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL EEG,PPG

C Photon momentum is fixed by the kinematics (two-body decay).
      PPG = 0.5*MUMMEV - 0.5*MUMMEV*R

C Photon energy.
      EEG = PPG

C Photon momentum.
      CALL PHOSGE(PPG)

C Photon position.
      CALL MEGVER(3)

C Putting GCODE(1) = 0 the positron is excluded.
      GCODE(1) = 0
      GCODE(2) = 1

      RETURN
      END

C ============================================================= C

      SUBROUTINE EPHRAD

C ============================================================= C
C PHOTONS from MU -> E+ GAMMA NU NUBAR (radiative decay).       C     
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      REAL EPOS,EPH,PHDIR(3)
      INTEGER I

C Energy and momemtum of outgoing photon.
      CALL EPHDEC

C Initial position of photon.
      CALL MEGVER(3)

C Putting GCODE(1) = 0 the positron is excluded.
      GCODE(1) = 0
      GCODE(2) = 1

      RETURN
      END

C ============================================================= C

      SUBROUTINE EPHANF

C ============================================================= C
C PHOTONS from E+-E- ANNIHILATION in flight.                    C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Photon energy and emission angle.
      CALL EACPHE(4)

C Initial position of photon.
      CALL MEGVER(3)

C Putting GCODE(1) = 0 the positron is excluded.
      GCODE(1) = 0
      GCODE(2) = 1
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPHFLA

C ============================================================= C
C PHOTONS with flat spectrum.                                   C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Photon energy and emission angle.
      CALL EFLPHE

C Initial position of photon.
      CALL MEGVER(3)

C Putting GCODE(1) = 0 the positron is excluded.
      GCODE(1) = 0
      GCODE(2) = 1
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPILRA

C ============================================================= C
C Pile-up photons with energy from radiative decay.             C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Photon energy and emission angle.
      CALL EPILPH(1)

C Initial position of photon.
      CALL MEGVER(4)

C Putting GCODE(1) = 1 && GCODE(2) = 1 both particles are photons.
      GCODE(1) = 1
      GCODE(2) = 1
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPILAF

C ============================================================= C
C Pile-up photons with energy from annihilation in flight.      C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Photon energy and emission angle.
      CALL EPILPH(2)

C Initial position of photon.
      CALL MEGVER(4)

C Putting GCODE(1) = 1 && GCODE(2) = 1 both particles are photons.
      GCODE(1) = 1
      GCODE(2) = 1
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPILRF

C ============================================================= C
C Pile-up photons: one from radiative decay and one from        C
C annihilation in flight.                                       C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Photon energy and emission angle.
      CALL EPILPH(3)

C Initial position of photon.
      CALL MEGVER(4)

C Putting GCODE(1) = 1 && GCODE(2) = 1 both particles are photons.
      GCODE(1) = 1
      GCODE(2) = 1
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPILFL

C ============================================================= C
C Pile-up photons with flat spectrum.                           C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Photon energy and emission angle.
      CALL EPILPH(4)

C Initial position of photon.
      CALL MEGVER(4)

C Putting GCODE(1) = 1 && GCODE(2) = 1 both particles are photons.
      GCODE(1) = 1
      GCODE(2) = 1
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE EPILPH(ICODE)

C ============================================================= C
C Energy and emission angles of pile-up photons.                C
C                                                               C
C ICODE = 1: both photons from radiative decay                  C
C ICODE = 2: both photons from annihilation in flight           C
C ICODE = 3: one photon from radiative decay and one from       C 
C            annihilation in flight                             C
C ICODE = 4: both photons with flat spectrum                    C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      INTEGER ICODE,J,I,K
      REAL EPH,EPOS,PHDIR(3),POSDIR(3),TPINIT(3)

C ============================================================= C
C Both photons from radiative decay.                            C
C ============================================================= C
      IF (ICODE.EQ.1) THEN

         DO J = 1,2

C Positron direction cosines.
            CALL ERDPOA(POSDIR)          
     
C Energy distribution of outgoing positron and photon.
            CALL ERDPOE(EPOS,EPH)

C Photon direction cosines.
            CALL ERDPHA(1,PHDIR)

C Photon momentum. 
            DO I = 1,3
               PINIT(J,I) = PHDIR(I)*EPH
            ENDDO
         ENDDO

C ============================================================= C
C Both photons from annihilation in flight.                     C
C ============================================================= C
      ELSE IF (ICODE.EQ.2) THEN

         DO J = 1,2
            CALL EPHANN(1)

C Store the value of the 2nd particle (photon) momentum.
            IF (J.EQ.1) THEN
               DO 2 I = 1,3
                  TPINIT(I) = PINIT(2,I)
 2             CONTINUE
            ENDIF
         ENDDO

C The first index is usually reserved to the positron.
         DO I = 1,3
            PINIT(1,I) = TPINIT(I)
         ENDDO

C ============================================================= C
C One photon from radiative decay and                           C
C one from annihilation in flight                               C
C ============================================================= C
      ELSE IF (ICODE.EQ.3) THEN

C 1st photon.
C Positron direction cosines.
         CALL ERDPOA(POSDIR)          
     
C Energy distribution of outgoing positron and photon.
         CALL ERDPOE(EPOS,EPH)

C Photon direction cosines.
         CALL ERDPHA(1,PHDIR)

C Photon momentum. 
         DO 3 I = 1,3
            PINIT(1,I) = PHDIR(I)*EPH
 3       CONTINUE

C 2nd photon.
         CALL EPHANN(1)

C ============================================================= C
C Both photons with flat spectrum.                              C
C ============================================================= C
      ELSE IF (ICODE.EQ.4) THEN

         DO J = 1,2
            CALL EFLPHE

C The first index is usually reserved to the positron.
            IF (J.EQ.1) THEN
               DO 4 I = 1,3
                  PINIT(1,I) = PINIT(2,I)
 4             CONTINUE
            ENDIF
         ENDDO
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE EFLPHE

C ============================================================= C
C Photons energy and momentum with flat spectrum.               C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL ARNDM
      REAL EEG,PPG,PPGM
      DATA PPGM/60./

C Magnitude of photon momentum.
      CALL GRNDM(ARNDM,1)
      PPG = PPGM*ARNDM

C Photon energy.
      EEG = PPG

C Photon momentum.
      CALL PHOSGE(PPG)

      RETURN
      END

C ================================================================== C

      SUBROUTINE MEGVER(INPCOD)

C ================================================================== C
C Creates a VERTEX inside the target with the correct distribution   C
C (see proposal, fig. 7 & 9 pagg. 10, 11)                            C
C ================================================================== C

      IMPLICIT NONE

#include"uniinc/megeve.inc"
#include"uniinc/megtar.inc"

      REAL VERTEX(3)
      REAL ARNDM,SX(3),AVEDEP,DELTA,DEPTH,ARNDM3(3)

      DATA SX/0.5,0.5,0.002/
      DATA AVEDEP/0.005/
      DATA DELTA/1.187/

      REAL XMAX,YMAX,ZMAX,XX(3)
      DATA XMAX,YMAX,ZMAX/50.,-30.,50./

      INTEGER I,J,INPCOD,ICASE,NUMED

C ================================================================== C
C ITAR   = 1: extended target                                        C 
C ITAR   = 0: pointlike target.                                      C
C INPCOD = 1: pair event with two particles in the same vertex (cor.)C
C INPCOD = 2: only positron                                          C
C INPCOD = 3: only photon                                            C
C INPCOD = 4: pair event, but any particle has its own vertex (unc.) C
C INPCOD = 5: positron event on chambers (noise).                    C
C ================================================================== C

 2    CONTINUE

      IF (ITAR.EQ.1) THEN

 1      CONTINUE
        CALL NORMCO(VERTEX(1),VERTEX(2),0.,0.,SX(1),SX(2),0.)
        CALL NORRAN(ARNDM)

        DEPTH = AVEDEP + ARNDM*SX(3)

        IF ((ABS(VERTEX(1)).GT.0.5*TARDIM(1)*COS(DELTA)).OR.
     +       (ABS(VERTEX(2)).GT.0.5*TARDIM(2)).OR.
     +       (DEPTH.LT.0).OR.
     +       (DEPTH.GT.TARDIM(3))) GOTO 1
      
        VERTEX(3) = VERTEX(1)*TAN(DELTA) + (DEPTH - 0.5*TARDIM(3))/ 
     +       COS(DELTA)
        
      ELSE IF (ITAR.EQ.0) THEN
        DO J = 1,3
          VERTEX(J) = 0.
        ENDDO
      ENDIF

      IF (INPCOD.EQ.1) THEN

C Pair event, one vertex only (correlated particles)
         DO I = 1,2
            DO J = 1,3
               VINIT(I,J) = VERTEX(J)
            ENDDO
         ENDDO

      ELSE IF (INPCOD.EQ.2) THEN

C Only positron.
         DO J = 1,3
            VINIT(1,J) = VERTEX(J)
            VINIT(2,J) = 0.
         ENDDO

      ELSE IF (INPCOD.EQ.3) THEN

C Only photon.
         DO J = 1,3
            VINIT(1,J) = 0.
            VINIT(2,J) = VERTEX(J)
         ENDDO

C Test
C         vinit(2,2) = +3.54
C         vertex(2)  = +3.54
C         vinit(2,3) = 0.
C         vertex(3)  = 0.

      ELSE IF (INPCOD.EQ.4) THEN

C ================================================================== C
C Pair event, but two distinct vertices (uncorrelated particles).    C
C ================================================================== C

C First vertex was already generated.
         IF (ICASE.EQ.0) THEN
            DO J = 1,3
               VINIT(1,J) = VERTEX(J)
            ENDDO
            ICASE = 1

C A new vertex must be generated.
            GOTO 2

C The second vertex was generated just before.
         ELSE IF (ICASE.EQ.1) THEN
            DO J = 1,3
               VINIT(2,J) = VERTEX(J)
            ENDDO
            ICASE = 0
         ENDIF

C Positron noise; the positron is generated on the chambers.
      ELSE IF (INPCOD.EQ.5) THEN
         DO J = 1,3
            VINIT(2,J) = 0.
         ENDDO

 3       CONTINUE
         CALL GRNDM(ARNDM3,3)
         VINIT(1,1) = XMAX*(2.*ARNDM3(1) - 1.)
         VINIT(1,2) = YMAX*ARNDM3(2)
         VINIT(1,3) = ZMAX*(2.*ARNDM3(3) - 1.)
         DO J = 1,3
            XX(J) = VINIT(1,J)
         ENDDO
         CALL GMEDIA(XX,NUMED)
         IF (NUMED.NE.42) GOTO 3

      ENDIF

      RETURN
      END

C =================================================================== C
C Routines for EXACT treatment of photons from RADIATIVE DECAY and    C 
C ANNIHILATION IN FLIGHT; they are in principle more time consuming.  C
C =================================================================== C

      REAL FUNCTION RADEC(YY)

C =================================================================== C
C Radiative decay.                                                    C
C =================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL YY
      REAL INVALP,NORM
      DATA INVALP/137.036/

      INTEGER ITIME
      DATA ITIME/0/

      IF (ITIME.EQ.0) THEN
         ITIME = 1
         NORM = 1./INVALP/3./PI
      ENDIF

      RADEC =  (1.- YY)*((2.*ALOG(EMMU/EMASS) - (17./6.) + 
     +    ALOG(1.- YY))*(3./YY - 2.*(1.- YY)**2) - 
     +         (1./12)*(1.- YY)*(22.- 13.*YY))

C This is for normalization purposes.
      RADEC = RADEC*NORM

      RETURN
      END

C =================================================================== C

      REAL FUNCTION FLANNI(YY)

C =================================================================== C
C ENERGY SPECTRUM for GAMMAS from ANNIHILATION IN FLIGHT of Michel    C
C positrons. This distribution is obtained folding the ANNIHILATION   C
C CROSS SECTION (FLIANN) with the POSITRON ENERGY DISTRIBUTION        C
C (MICHEL) and the ELECTRON DENSITY of the materials crossed by the   C
C positron (DENS).                                                    C
C =================================================================== C

      IMPLICIT NONE

      REAL XX,YY,SUM,DX
      REAL MICHEL,DENS,FLIANN
      INTEGER I

      SUM = 0.
      DX =.001
      DO I = 1,1000
         XX = FLOAT(I - 1)*DX + 0.5*DX
         SUM = SUM + MICHEL(XX)*FLIANN(XX,YY)*DX*DENS(XX)
      ENDDO

      FLANNI = SUM

      RETURN
      END

C =================================================================== C

      REAL FUNCTION FLIANN(XX,YY)

C =================================================================== C
C POSITRON ANNIHILATION IN FLIGHT CROSS SECTION. QUASI-FORWARD        C 
C PHOTONS !                                                           C
C Reference: Yung-Su Tsai Rev.Mod.Phys. 46 832 (1974)                 C
C =================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megeve.inc"

      REAL XX,YY,NORM,NORM1
      DATA NORM/3.86E-11/
      REAL E,GGAMMA,ZXY
      REAL INVALP
      DATA INVALP/137.036/
      INTEGER ITIME
      DATA ITIME/0/

      IF (ITIME.EQ.0) THEN
         ITIME = 1
         NORM1 = PI/INVALP/INVALP
      ENDIF

C Protection.
      IF (YY.EQ.0.) THEN
        FLIANN = 0.
        RETURN
      ENDIF

      E = XX*EMAX
      ZXY = XX*YY

      GGAMMA = E/EMMEV
      FLIANN = (NORM**2)*(1./GGAMMA**2)*
     +         (ZXY/(ZXY/(2.*GGAMMA) + (1.- ZXY)) +
     +         (ZXY/(2.*GGAMMA)     + (1. - ZXY))/ZXY)

C Cross section is given in cm. 
      FLIANN = FLIANN*NORM1/EMMEV

      RETURN
      END

C =================================================================== C

      REAL FUNCTION DENS(XX)

C =================================================================== C
C ELECTRON AVERAGE DENSITY in crossed materials (only on TARGET).     C
C =================================================================== C

      IMPLICIT NONE

      REAL XX
      REAL EDEN,WTAR,EPATH
      DATA EDEN,WTAR/0.5875E+24,0.1/

      DENS = EDEN*EPATH(XX)/WTAR

      RETURN
      END
 
C =================================================================== C

      REAL FUNCTION EPATH(XX)

C =================================================================== C
C ELECTRON PATH in the target as a function of ENERGY.                C
C Probably this function must be computed again, since the            C
C distribution of the muon stopping points in the target (Proposal,   C
C fig. 9) was misunderstood.                                          C
C =================================================================== C

      IMPLICIT NONE

      REAL XX

      REAL X1,X2
      DATA X1,X2/8.,30./

      REAL P1,P2,P3
      DATA P1,P2,P3/0.0207,0.0154,-0.00163/

      REAL PP1,PP2
      DATA PP1,PP2/-3.075,-0.01327/

      REAL PPP1,PPP2,PPP3
      DATA PPP1,PPP2,PPP3/0.001434,0.001451,-1.628E-5/

      IF (XX.LE.X1) THEN
         EPATH = P1 + P2*XX + P3*XX*XX
      ELSE IF (XX.GT.X1.AND.XX.LE.X2) THEN
         EPATH = EXP(PP1 + PP2*XX)
      ELSE IF (XX.GT.X2) THEN
         EPATH = PPP1 + PPP2*XX + PPP3*XX*XX
      ENDIF

      RETURN
      END










