
      SUBROUTINE MEGLIX

C ===================================================================== C
C Liquid Xenon detector.                                                C
C                                                                       C
C History:                                                              C
C --------                                                              C
C 18/09/00   :  Creation of the file                                    C
C                                                                       C
C ===================================================================== C
  
      IMPLICIT NONE
      
      CALL LIXMAT
      CALL LIXMED
      CALL LIXVOL
      CALL LIXPOS
      CALL LIXDET

      RETURN
      END

C ===================================================================== C

      SUBROUTINE MEGLIR

C ===================================================================== C
C Liquid Xenon detector.                                                C
C                                                                       C
C History:                                                              C
C --------                                                              C
C 18/09/00   :  Creation of the file                                    C
C                                                                       C
C ===================================================================== C
  
      IMPLICIT NONE
      
      CALL LIXMAT
      CALL LIXMED
      CALL LIXVOL
      CALL LIRPOS
      CALL LIXDET

      RETURN
      END

C ===================================================================== C

      SUBROUTINE LIXMAT

C ===================================================================== C
C Define materials used in the liquid Xenon calorimeter.                C
C ===================================================================== C

      IMPLICIT NONE

#include"uniinc/meglix.inc"

C PET Polyethylene.
      REAL APET(2),ZPET(2),WPET(2),DPET
      DATA APET/12.011,1.008/
      DATA ZPET/6.,1./
      DATA WPET/1.,2./
      DATA DPET/.94/	

C PMT photomultiplier (this is a mixture of glass and copper with the   
C appropriate density to take into account the vacuum). 
      REAL APMT(3),ZPMT(3),WPMT(3),DPMT
      REAL ROVE, ROCU, TKVE, TKCU
      DATA ROVE/2.3/, ROCU/8.9/, TKVE/.6/, TKCU/.2/
      DATA APMT/16.0,28.0,63.5/
      DATA ZPMT/8.,14.,29./

      REAL SIABSL, UBUF, MPMT
      DATA SIABSL/0./, UBUF/0./

C LiXe (Liquid Xenon).
      CALL GSMATE(22,'Liquid Xenon    $',
C             A     Z    DEN  RLEN  ALEN    
     +     131.29, 54.0, 3.0, 2.77,  1.8, UBUF, 1)

C LiXe (GAS Xenon).
      CALL GSMATE(23,'GAS Xe 300K 1.4atm $',
C             A     Z    DEN        RLEN  ALEN    
     +     131.29, 54.0, 0.0082, 1033.64,  1.8, UBUF, 1)

C LiXe (GAS Xenon).
      CALL GSMATE(27,'GAS Xe 170K 1.4atm $',
C             A     Z    DEN        RLEN  ALEN    
     +     131.29, 54.0, 0.011,  771.00,  1.8, UBUF, 1)

C PET (Polyethylene). 
      CALL GSMIXT(25,'PET per LiXe    $',APET,ZPET,DPET,-2,WPET)

C PMT (Photomultiplier). 
      MPMT    = TKVE*ROVE + TKCU*ROCU
      WPMT(1) = TKVE*ROVE*2.*APMT(1)/MPMT/(APMT(2) + 2.*APMT(1))
      WPMT(2) =    TKVE*ROVE*APMT(2)/MPMT/(APMT(2) + 2.*APMT(1))
      WPMT(3) = TKCU*ROCU/MPMT
      DPMT    = MPMT/LXPMT
      CALL GSMIXT(26,'XPMT    $',APMT,ZPMT,DPMT,3,WPMT)

      RETURN
      END

C ===================================================================== C

      SUBROUTINE LIXMED

C ===================================================================== C
C Define the tracking medium parameters, like                           C
C     IFIELD = presence of magnetic field                               C
C     TMAXFD = maximum angular deviation                                C
C     STEMAX = maximum step permitted                                   C
C     DEEMAX = maximum fractional energy loss                           C
C ===================================================================== C
     
      IMPLICIT NONE

      INTEGER IFIELD
      REAL    FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN

#include"uniinc/meglix.inc"

C Presence of magnetic field in the tracking medium.  
      IFIELD = 1

C Maximum magnetic field. 
      FIELDM = 15.

C Maximum angular deviation.  
      TMAXFD = 20.0  

C Maximum step permitted (cm).
      STEMAX = 2.0

C Maximum fractional energy loss.
      DEEMAX = 0.01               

C Boundary crossing precision.
      EPSIL = 0.002 

C Minimum value for the maximum step imposed by dedx/mult scatt etc.
      STMIN = 0.001
                                                                 
C ===================================================================== C
C Definitions of tracking media.                                        C
C ===================================================================== C
   
C Tracking medium 21
      CALL GSTMED(21,'LiXe Aluminium     $', 9,                               
     +       1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

C Tracking medium 22 LiXe 
      CALL GSTMED(22,'Liquid Xenon       $',22,                               
     +       1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

C Tracking medium 23 Gas Xe 
      CALL GSTMED(23,'Gas Xenon          $',27,                               
     +       1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

C Tracking medium 24
      CALL GSTMED(24,'LiXe Vacuum        $',16,                               
     +      -1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

C Tracking medium 25 LiXe PMT support (PET).
      CALL GSTMED(25,'LiXe PMT support   $',25,            
     +       1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

C Tracking medium 26 LiXe PMT.
      CALL GSTMED(26,'LiXe PMT mixture   $',26,            
     +      -1,IFIELD,FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

      RETURN
      END

C ===================================================================== C

      SUBROUTINE LIXVOL

C ===================================================================== C
C Define detector volumes by calling the routine                        C
C   GSVOLU (NAME,SHAPE,NTRKGMED,PAR,NPAR,ISYSTEMVOLNUM)                 C
C ===================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/meglix.inc"

      INTEGER IVOLU      

C Dimensions of the detector volumes. 
      REAL PLXE(5),PAL1(5),PVAC(5),PAL2(5)
      REAL PPMT(3),PPLA(5) 
      REAL XEVOLU 
                                        
C ===================================================================== C
C Liquid Xenon calorimeter.                                             C
C ===================================================================== C

C PMTs.
      PPMT(1) = 0.
      PPMT(2) = LXPMR
      PPMT(3) = 0.5*LXPMT 
      WRITE(*,*) 'XPMT : ',PPMT
      CALL GSVOLU('XPMT','TUBE',26,PPMT,3,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XPMT) '
        STOP' Abnormal End '
      ENDIF

C LiXe Barrel: Liquid Xenon volume.
      PLXE(1) =     LXRIN 
      PLXE(2) =     LXREX 
      PLXE(3) =  .5*LXZTO 
      PLXE(4) = -.5*LXFIT  
      PLXE(5) = . 5*LXFIT 
      XEVOLU = PI*(LXREX**2 - LXRIN**2)*LXZTO*LXFIT/360./1000.
      WRITE(*,*) ' Xenon Volume (l) : ',XEVOLU
      WRITE(*,*) ' XLIQ : ',PLXE
      CALL GSVOLU('XLIQ','TUBS',22,PLXE,5,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XLIQ) '
        STOP' Abnormal End '
      ENDIF

C LiXe Barrel: PMT plastic containers.
      PPLA(1) =     LXRIN - LXPLA
      PPLA(2) =     LXREX + LXPLA
      PPLA(3) =  .5*LXZTO + LXPLA 
      PPLA(4) = -.5*LXFIT  
      PPLA(5) =  .5*LXFIT
      WRITE(*,*) 'XPLA : ',PPLA
      CALL GSVOLU('XPLA','TUBS',25,PPLA,5,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XPLA) '
        STOP' Abnormal End '
      ENDIF

C LiXe Barrel: inner Al container.
      PAL1(1) =     LXRIN - LXPLA - LXALI
      PAL1(2) =     LXREX + LXPLA + LXALI
      PAL1(3) =  .5*LXZTO + LXPLA + LXALI
      PAL1(4) = -.5*LXFIT  
      PAL1(5) =  .5*LXFIT 
      WRITE(*,*) 'XALI : ',PAL1
      CALL GSVOLU('XALI','TUBS',21,PAL1,5,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XALI) '
        STOP' Abnormal End '
      ENDIF

C LiXe Barrel: vacuum chamber.
      PVAC(1) =     LXRIN - LXPLA - LXALI - LXVAC
      PVAC(2) =     LXREX + LXPLA + LXALI + LXVAC
      PVAC(3) =  .5*LXZTO + LXPLA + LXALI + LXVAC
      PVAC(4) = -.5*LXFIT 
      PVAC(5) =  .5*LXFIT 
      WRITE(*,*) 'XVAC : ',PVAC
      CALL GSVOLU('XVAC','TUBS',24,PVAC,5,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XVAC) '
        STOP' Abnormal End '
      ENDIF

C Li Xe Barrel: external Al container.
      PFBA(1) =     LXRIN - LXPLA - LXALI - LXVAC - LXALE
      PFBA(2) =     LXREX + LXPLA + LXALI + LXVAC + LXALE
      PFBA(3) =  .5*LXZTO + LXPLA + LXALI + LXVAC + LXALE
      PFBA(4) = -.5*LXFIT  
      PFBA(5) =  .5*LXFIT 
      WRITE(*,*) 'XALE : ',PFBA
      CALL GSVOLU('XALE','TUBS',21,PFBA,5,IVOLU)
      IF (IVOLU.LE.0) THEN 
        WRITE(*,*) ' Error calling GSVOLU(XALE) '
        STOP' Abnormal End '
      ENDIF

C XFRA.
      PFBA(1) = PFBA(1) - 0.1
      PFBA(2) = PFBA(2) + 10.
      PFBA(3) = PFBA(3) + 10.
      PFBA(4) = -90.
      PFBA(5) = +90.
      CALL GSVOLU('XFRA','TUBS',24,PFBA,5,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XFRA) '
        STOP' Abnormal End '
      ENDIF

C ===================================================================== C
C Liquid Xe: end cup UP.                                                C
C ===================================================================== C

C LiXe Endcup: plastic container.
      PFEC(1) = .5*(LXREX - LXRIN) + LXPLA
      PFEC(2) = .5*LXPLA
      PFEC(3) = .5*LXZTO + LXPLA 
      WRITE(*,*) 'XPLU : ',PFEC
      CALL GSVOLU('XPLU','BOX ',25,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN 
        WRITE(*,*) ' Error calling GSVOLU(XPLU) '
        STOP' Abnormal End '
      ENDIF

C LiXe Endcup: Inner Al container.
      PFEC(1) = .5*(LXREX - LXRIN) + LXPLA + LXALI
      PFEC(2) = .5*(LXPLA + LXALI)
      PFEC(3) = .5*LXZTO + LXPLA + LXALI 
      WRITE(*,*) 'XAEU : ',PFEC
      CALL GSVOLU('XAEU','BOX ',21,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN 
        WRITE(*,*) ' Error calling GSVOLU(XAEU) '
        STOP' Abnormal End '
      ENDIF

C LiXe Endcup: Vacuum chamber.
      PFEC(1) = .5*(LXREX - LXRIN) + LXPLA + LXALI + LXVAC
      PFEC(2) = .5*(LXPLA + LXALI + LXVAC)
      PFEC(3) = .5*LXZTO + LXPLA + LXALI + LXVAC 
      WRITE(*,*) 'XVAU : ',PFEC
      CALL GSVOLU('XVAU','BOX ',24,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XVAU) '
        STOP' Abnormal End '
      ENDIF

C LiXe Endcup: External Al container.
      PFEC(1)= .5*(LXREX - LXRIN) + LXPLA + LXALI + LXVAC + LXALE
      PFEC(2)= .5*(LXPLA + LXALI + LXVAC + LXALE)
      PFEC(3)= .5*LXZTO + LXPLA + LXALI + LXVAC + LXALE  
      WRITE(*,*) 'XAUU : ',PFEC
      CALL GSVOLU('XAUU','BOX ',21,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN 
        WRITE(*,*) ' Error calling GSVOLU(XAUU) '
        STOP' Abnormal End '
      ENDIF

C ===================================================================== C
C Liquid Xe: end cup DOWN.                                              C
C ===================================================================== C

C LiXe Endcup: plastic container.
      PFEC(1) = .5*(LXREX - LXRIN) + LXPLA
      PFEC(2) = .5*LXPLA
      PFEC(3) = .5*LXZTO + LXPLA 
      WRITE(*,*) 'XPLD : ',PFEC
      CALL GSVOLU('XPLD','BOX ',25,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XPLD) '
        STOP' Abnormal End '
      ENDIF

C LiXe Endcup: Inner Al container.
      PFEC(1) = .5*(LXREX - LXRIN) + LXPLA + LXALI
      PFEC(2) = .5*(LXPLA + LXALI)
      PFEC(3) = .5*LXZTO + LXPLA + LXALI 
      WRITE(*,*) 'XAED : ',PFEC
      CALL GSVOLU('XAED','BOX ',21,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XAED) '
        STOP' Abnormal End '
      ENDIF

C LiXe Endcup: Vacuum chamber.
      PFEC(1) = .5*(LXREX - LXRIN) + LXPLA + LXALI + LXVAC
      PFEC(2) = .5*(LXPLA + LXALI + LXVAC)
      PFEC(3) = .5*LXZTO + LXPLA + LXALI + LXVAC 
      WRITE(*,*) 'XVAD : ',PFEC
      CALL GSVOLU('XVAD','BOX ',24,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN
        WRITE(*,*) ' Error calling GSVOLU(XVAD) '
        STOP' Abnormal End '
      ENDIF

C LiXe Endcup: External Al container.
      PFEC(1) = .5*(LXREX - LXRIN) + LXPLA + LXALI + LXVAC + LXALE
      PFEC(2) = .5*(LXPLA + LXALI + LXVAC + LXALE)
      PFEC(3) = .5*LXZTO + LXPLA + LXALI + LXVAC + LXALE  
      WRITE(*,*) 'XADD : ',PFEC
      CALL GSVOLU('XADD','BOX ',21,PFEC,3,IVOLU)
      IF (IVOLU.LE.0) THEN 
        WRITE(*,*)' Error calling GSVOLU(XADD) '
        STOP' Abnormal End '
      ENDIF

      RETURN
      END

C ===================================================================== C

      SUBROUTINE LIXPOS

C ===================================================================== C
C Volume positioning  by calling the routine                            C
C   GSPOS(NAME,COPYNUMBER,MOTHER,POSITIONVECTOR,IROT,KONLYTAG)          C
C ===================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/meglix.inc"

      REAL PMTPHI

C This common block is retained for backward compatibility with LIXPMT 
C and to put the PMTs in correct position.
      COMMON/PMTROT/PMTPHI(NPMTT)

      INTEGER I,I1,I2,II
      REAL XTRA,YTRA,TE1,FI1,TE2,FI2,TE3,FI3,DX,DY,DZ
      DATA TE1/90./,TE2/90./,TE3/0./,FI3/0./

C Compute the PMT positions relative to the LiXe. 
      CALL LIXPMT

C XLIQ: liquid Xenon into XPLA.
      CALL GSPOS('XLIQ',1,'XPLA',0.,0.,0.,0,'ONLY')

C XPMT into XPLA.

C Face I.
      I1 = 1
      I2 = NPMTI
      DO I = I1,I2
         DX = COS((PMTPHI(I) - 180.)/57.29)*0.5*LXPMT 
         DY = SIN((PMTPHI(I) - 180.)/57.29)*0.5*LXPMT 
         DZ = 0.
         CALL GSROTM(100 + I,0.,0.,90.,90.+ PMTPHI(I),90.,PMTPHI(I))
         CALL GSPOS('XPMT',100 + I,'XPLA',XIPMT(I) - DX,
     +        YIPMT(I) - DY,
     +        ZIPMT(I),100 + I,'ONLY')
      ENDDO

C Face O.
      I1 = NPMTI + 1
      I2 = NPMTI + NPMTO
      DO I = I1,I2
         DX = COS(PMTPHI(I)/57.29)*0.5*LXPMT
         DY = SIN(PMTPHI(I)/57.29)*0.5*LXPMT
         DZ = 0.
         CALL GSROTM(100 + I,0.,0.,90.,90.+ PMTPHI(I),90.,PMTPHI(I))
         CALL GSPOS('XPMT',100 + I,'XPLA',XOPMT(I - I1 + 1) + DX,
     +        YOPMT(I - I1 + 1) + DY,
     +        ZOPMT(I - I1 + 1),
     +        100 + I,'ONLY')
      ENDDO

C Face U.
      I1 = NPMTI + NPMTO + 1
      I2 = NPMTI + NPMTO + NPMTC
      II = 0
      DO I = I1,I2
         II = II + 1
         DX = -.5*(LXREX - LXRIN) + FLOAT(II - 1)*PDR + 0.5*PDR
         IF (II.EQ.NPMCR) II = 0
         CALL GSROTM(100 + I,90.,0.,0.,0.,90.,270.)
         CALL GSPOS('XPMT',100 + I,'XPLU',DX,
     +        0.,
     +        ZUPMT(I - I1 + 1),
     +        100 + I,'ONLY')
      ENDDO
      
C Face D.  
      I1 = NPMTI + NPMTO + NPMTC + 1
      I2 = NPMTI + NPMTO + 2*NPMTC
      II = 0
      DO I = I1,I2
         II = II + 1
         DX =-.5*(LXREX - LXRIN) + FLOAT(II - 1)*PDR + 0.5*PDR
         IF (II.EQ.NPMCR) II = 0
         CALL GSROTM(100 + I,90.,0.,0.,0.,90.,270.)
         CALL GSPOS('XPMT',100 + I,'XPLD',DX,
     +        0.,
     +        ZDPMT(I - I1 + 1),
     +        100 + I,'ONLY')
      ENDDO

C Face L.
      I1 = NPMTI + NPMTO + 2*NPMTC + 1
      I2 = NPMTI + NPMTO + 2*NPMTC + NPMTE
      DO I = I1,I2
         CALL GSPOS('XPMT',100 + I,'XPLA',XLPMT(I - I1 + 1),
     +        YLPMT(I - I1 + 1),
     +        ZLPMT(I - I1 + 1) - 0.5*LXPMT
     +        ,0,'ONLY')
      ENDDO
      
C Face R.
      I1 = NPMTI + NPMTO + 2*NPMTC + NPMTE + 1
      I2 = NPMTI + NPMTO + 2*NPMTC + 2*NPMTE
      DO I = I1,I2
         CALL GSPOS('XPMT',100 + I,'XPLA',XRPMT(I - I1 + 1),
     +        YRPMT(I - I1 + 1),
     +        ZRPMT(I - I1 + 1) + 0.5*LXPMT
     +        ,0,'ONLY')
      ENDDO

C Compute the true coordinates of the PMTs (to be used 
C for calculating the expected signal).
      CALL LIXTPM

C XPLA: plastic support for the PMT.
      CALL GSPOS('XPLA',1,'XALI',0.,0.,0.,0,'ONLY')

C XALI: inner Al container.
      CALL GSPOS('XALI',1,'XVAC',0.,0.,0.,0,'ONLY')

C XVAC: vacuum chamber.
      CALL GSPOS('XVAC',1,'XALE',0.,0.,0.,0,'ONLY')

C XALE: external Al container into XFRA.
      CALL GSPOS('XALE',1,'XFRA',0.,0.,0.,0,'ONLY')

C XPLU: Plastic PMT support.
      YTRA = -.5*LXALI
      CALL GSPOS('XPLU',1,'XAEU',0.,YTRA,0.,0,'ONLY')

C XAEU: Inner Al container.
      YTRA = -.5*LXVAC
      CALL GSPOS('XAEU',1,'XVAU',0.,YTRA,0.,0,'ONLY')

C XVAU: Inner Al container.
      YTRA = -.5*LXALE
      CALL GSPOS('XVAU',1,'XAUU',0.,YTRA,0.,0,'ONLY')

C XAUU: End cup UP into XFRA.
      FI1 = .5*LXFIT
      FI2 = FI1 + 90. 
      CALL GSROTM(1,TE1,FI1,TE2,FI2,TE3,FI3,)
      XTRA = .5*(LXREX + LXRIN)*COS(LXFIT*TWOPI/720.) -
     +   PFEC(2)*SIN(LXFIT*TWOPI/720.) 
      YTRA = .5*(LXREX + LXRIN)*SIN(LXFIT*TWOPI/720.) +
     +   PFEC(2)*COS(LXFIT*TWOPI/720.) 
      CALL GSPOS('XAUU',3,'XFRA',XTRA,YTRA,0.,1,'ONLY')

C XPLD: Plastic PMT support.
      YTRA = -.5*LXALI
      CALL GSPOS('XPLD',1,'XAED',0.,YTRA,0.,0,'ONLY')

C XAED: Inner Al container.
      YTRA = -.5*LXVAC
      CALL GSPOS('XAED',1,'XVAD',0.,YTRA,0.,0,'ONLY')

C XVAD: Inner Al container.
      YTRA = -.5*LXALE
      CALL GSPOS('XVAD',1,'XADD',0.,YTRA,0.,0,'ONLY')

C XADD: End cup UP into XFRA.
      FI1 = 180. -.5*LXFIT
      FI2 = FI1 + 90. 
      CALL GSROTM(2,TE1,FI1,TE2,FI2,TE3,FI3,)
      XTRA = .5*(LXREX + LXRIN)*COS(LXFIT*TWOPI/720.) -
     +   PFEC(2)*SIN(LXFIT*TWOPI/720.) 
      YTRA = .5*(LXREX + LXRIN)*SIN(LXFIT*TWOPI/720.) +
     +   PFEC(2)*COS(LXFIT*TWOPI/720.) 
      YTRA = -YTRA 
      CALL GSPOS('XADD',4,'XFRA',XTRA,YTRA,0.,2,'ONLY')

C XFRA: Liquid Xenon frame into FRAM.
      CALL GSROTM(2000,90.,180.,90.,90.,180.,0.)
      CALL GSPOS('XFRA',1,'FRAM',0.,0.,0.,2000,'ONLY')

      RETURN
      END

C ===================================================================== C

      SUBROUTINE LIXTPM

C ===================================================================== C
C Transform the PMT coordinates from the original reference frame to    C
C the mother frame. These new coordinates are used for computing the    C
C expected PMT signal.                                                  C
C ===================================================================== C

      IMPLICIT NONE

#include"uniinc/meglix.inc"

      INTEGER JPMT

      WRITE(68,*) ' True PMT coordinates (absolute frame) '

C Inner Face.
      DO JPMT = 1,NPMTI
         XITPMT(JPMT) = - XIPMT(JPMT) 
         YITPMT(JPMT) = - YIPMT(JPMT) 
         ZITPMT(JPMT) = + ZIPMT(JPMT)
         WRITE(68,*) JPMT,XITPMT(JPMT),YITPMT(JPMT),ZITPMT(JPMT)
      ENDDO

C Outer Face.
      DO JPMT = 1,NPMTO
         XOTPMT(JPMT) = - XOPMT(JPMT) 
         YOTPMT(JPMT) = - YOPMT(JPMT) 
         ZOTPMT(JPMT) = + ZOPMT(JPMT) 
         WRITE(68,*) JPMT,XOTPMT(JPMT),YOTPMT(JPMT),ZOTPMT(JPMT)
      ENDDO

C Upper Cover.
      DO JPMT = 1,NPMTC
         XUTPMT(JPMT) = - XUPMT(JPMT) 
         YUTPMT(JPMT) = - YUPMT(JPMT) 
         ZUTPMT(JPMT) = + ZUPMT(JPMT) 
         WRITE(68,*) JPMT,XUTPMT(JPMT),YUTPMT(JPMT),ZUTPMT(JPMT)
      ENDDO

C Down Cover.
      DO JPMT = 1,NPMTC
         XDTPMT(JPMT) = - XDPMT(JPMT) 
         YDTPMT(JPMT) = - YDPMT(JPMT) 
         ZDTPMT(JPMT) = + ZDPMT(JPMT)
         WRITE(68,*) JPMT,XDTPMT(JPMT),YDTPMT(JPMT),ZDTPMT(JPMT) 
      ENDDO

C Left End
      DO JPMT = 1,NPMTE
         XLTPMT(JPMT) = - XLPMT(JPMT) 
         YLTPMT(JPMT) = - YLPMT(JPMT) 
         ZLTPMT(JPMT) = + ZLPMT(JPMT)
         WRITE(68,*) JPMT,XLTPMT(JPMT),YLTPMT(JPMT),ZLTPMT(JPMT) 
      ENDDO

C Right End
      DO JPMT = 1,NPMTE
         XRTPMT(JPMT) = - XRPMT(JPMT) 
         YRTPMT(JPMT) = - YRPMT(JPMT) 
         ZRTPMT(JPMT) = + ZRPMT(JPMT)
         WRITE(68,*) JPMT,XRTPMT(JPMT),YRTPMT(JPMT),ZRTPMT(JPMT) 
      ENDDO

      CLOSE (68)

      RETURN
      END

C ===================================================================== C

      SUBROUTINE LIRPOS

C ===================================================================== C
C Volume positioning  by calling the routine                            C
C   GSPOS(NAME,COPYNUMBER,MOTHER,POSITIONVECTOR,IROT,KONLYTAG)          C
C ===================================================================== C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/meglix.inc"

      REAL PMTPHI

C This common block is retained for backward compatibility with LIXPMT 
C and to put the PMTs in correct position.
      COMMON/PMTROT/PMTPHI(NPMTT)

      INTEGER I,I1,I2,II
      REAL XTRA,YTRA,TE1,FI1,TE2,FI2,TE3,FI3,DX,DY
      DATA TE1/90./,TE2/90./,TE3/0./,FI3/0./

C Compute the PMT positions relative to the LiXe. 
      CALL LIXPMT

C XLIQ: liquid Xenon into XPLA.
      CALL GSPOS('XLIQ',1,'XPLA',0.,0.,0.,0,'ONLY')

C XPMT into XPLA.

C Face I.
      I1 = 1
      I2 = NPMTI
      DO I = I1,I2
         DX = COS((PMTPHI(I) - 180.)/57.29)*0.5*LXPMT 
         DY = SIN((PMTPHI(I) - 180.)/57.29)*0.5*LXPMT 
         CALL GSROTM(100 + I,0.,0.,90.,90.+ PMTPHI(I),90.,PMTPHI(I))
         CALL GSPOS('XPMT',100 + I,'XPLA',XIPMT(I) - DX,
     +        YIPMT(I) - DY,
     +        ZIPMT(I),100 + I,'ONLY')
      ENDDO

C Face O.
      I1 = NPMTI + 1
      I2 = NPMTI + NPMTO
      DO I = I1,I2
         DX = COS(PMTPHI(I)/57.29)*0.5*LXPMT
         DY = SIN(PMTPHI(I)/57.29)*0.5*LXPMT 
         CALL GSROTM(100 + I,0.,0.,90.,90.+ PMTPHI(I),90.,PMTPHI(I))
         CALL GSPOS('XPMT',100 + I,'XPLA',XOPMT(I - I1 + 1) + DX,
     +        YOPMT(I - I1 + 1) + DY,
     +        ZOPMT(I - I1 + 1),
     +        100 + I,'ONLY')
      ENDDO

C Face U.
      I1 = NPMTI + NPMTO + 1
      I2 = NPMTI + NPMTO + NPMTC
      II = 0
      DO I = I1,I2
         II = II + 1
         DX = -.5*(LXREX - LXRIN) + FLOAT(II - 1)*PDR + 0.5*PDR
         IF (II.EQ.NPMCR) II = 0
         CALL GSROTM(100 + I,90.,0.,0.,0.,90.,270.)
         CALL GSPOS('XPMT',100 + I,'XPLU',DX,
     +        0.,
     +        ZUPMT(I - I1 + 1),
     +        100 + I,'ONLY')
      ENDDO
      
C Face D.  
      I1 = NPMTI + NPMTO + NPMTC + 1
      I2 = NPMTI + NPMTO + 2*NPMTC
      II = 0
      DO I = I1,I2
         II = II + 1
         DX =-.5*(LXREX - LXRIN) + FLOAT(II - 1)*PDR + 0.5*PDR
         IF (II.EQ.NPMCR) II = 0
         CALL GSROTM(100 + I,90.,0.,0.,0.,90.,270.)
         CALL GSPOS('XPMT',100 + I,'XPLD',DX,
     +        0.,
     +        ZUPMT(I - I1 + 1),
     +        100 + I,'ONLY')
      ENDDO

C Face L.
      I1 = NPMTI + NPMTO + 2*NPMTC + 1
      I2 = NPMTI + NPMTO + 2*NPMTC + NPMTE
      DO I = I1,I2
         CALL GSPOS('XPMT',100 + I,'XPLA',XLPMT(I - I1 + 1),
     +        YLPMT(I - I1 + 1),
     +        ZLPMT(I - I1 + 1) - 0.5*LXPMT
     +        ,0,'ONLY')
      ENDDO
      
C Face R.
      I1 = NPMTI + NPMTO + 2*NPMTC + NPMTE + 1
      I2 = NPMTI + NPMTO + 2*NPMTC + 2*NPMTE
      DO I = I1,I2
         CALL GSPOS('XPMT',100 + I,'XPLA',XRPMT(I - I1 + 1),
     +        YRPMT(I - I1 + 1),
     +        ZRPMT(I - I1 + 1) + 0.5*LXPMT
     +        ,0,'ONLY')
      ENDDO

C XPLA: plastic support for the PMT.
      CALL GSPOS('XPLA',1,'XALI',0.,0.,0.,0,'ONLY')

C XALI: inner Al container.
      CALL GSPOS('XALI',1,'XVAC',0.,0.,0.,0,'ONLY')

C XVAC: vacuum chamber.
      CALL GSPOS('XVAC',1,'XALE',0.,0.,0.,0,'ONLY')

C XALE: external Al container into XFRA.
      CALL GSPOS('XALE',1,'XFRA',0.,0.,0.,0,'ONLY')

C XPLU: Plastic PMT support.
      YTRA = -.5*LXALI
      CALL GSPOS('XPLU',1,'XAEU',0.,YTRA,0.,0,'ONLY')

C XAEU: Inner Al container.
      YTRA = -.5*LXVAC
      CALL GSPOS('XAEU',1,'XVAU',0.,YTRA,0.,0,'ONLY')

C XVAU: Inner Al container.
      YTRA = -.5*LXALE
      CALL GSPOS('XVAU',1,'XAUU',0.,YTRA,0.,0,'ONLY')

C XAUU: End cup UP into XFRA.
      FI1 = .5*LXFIT
      FI2 = FI1 + 90. 
      CALL GSROTM(1,TE1,FI1,TE2,FI2,TE3,FI3,)
      XTRA = .5*(LXREX + LXRIN)*COS(LXFIT*TWOPI/720.) -
     +   PFEC(2)*SIN(LXFIT*TWOPI/720.) 
      YTRA = .5*(LXREX + LXRIN)*SIN(LXFIT*TWOPI/720.) +
     +   PFEC(2)*COS(LXFIT*TWOPI/720.) 
      CALL GSPOS('XAUU',3,'XFRA',XTRA,YTRA,0.,1,'ONLY')

C XPLD: Plastic PMT support.
      YTRA = -.5*LXALI
      CALL GSPOS('XPLD',1,'XAED',0.,YTRA,0.,0,'ONLY')

C XAED: Inner Al container.
      YTRA = -.5*LXVAC
      CALL GSPOS('XAED',1,'XVAD',0.,YTRA,0.,0,'ONLY')

C XVAD: Inner Al container.
      YTRA = -.5*LXALE
      CALL GSPOS('XVAD',1,'XADD',0.,YTRA,0.,0,'ONLY')

C XADD: End cup UP into XFRA.
      FI1 = 180. -.5*LXFIT
      FI2 = FI1 + 90. 
      CALL GSROTM(2,TE1,FI1,TE2,FI2,TE3,FI3,)
      XTRA = .5*(LXREX + LXRIN)*COS(LXFIT*TWOPI/720.) -
     +   PFEC(2)*SIN(LXFIT*TWOPI/720.) 
      YTRA = .5*(LXREX + LXRIN)*SIN(LXFIT*TWOPI/720.) +
     +   PFEC(2)*COS(LXFIT*TWOPI/720.) 
      YTRA = -YTRA 
      CALL GSPOS('XADD',4,'XFRA',XTRA,YTRA,0.,2,'ONLY')

C XFRA: Liquid Xenon frame into FRAM.
      CALL GSROTM(2000,90.,180.,90.,90.,180.,0.)
C      CALL GSPOS('XFRA',1,'FRAM',0.,0.,0.,2000,'ONLY')
C
C Small correction for roberto.
      CALL GSPOS('XFRA',1,'FRAM',0.,0.,0.,0,'ONLY')

      RETURN
      END

C ===================================================================== C

      SUBROUTINE LIXPMT

C ===================================================================== C
C Define the PMT positions (origin in 0,0,0).                           C
C ===================================================================== C

      IMPLICIT NONE

#include"uniinc/meglix.inc"              
      
      REAL DANGI,DANGO,Z,DZ,ANG,AA,R,DR
      INTEGER KI,KO,KU,KD,KL,KR,IZ,IR,NNTT,KK,I
      REAL FAC

C PMT order in PMTPHI: i,o,u,d,l,r as faces 1,2,3,4,5,6
      REAL PMTPHI
      COMMON/PMTROT/PMTPHI(NPMTT)

      DATA FAC/0.017453293/

      WRITE(*,*)'LIXPMT: projet n. of PMTs ',NPMTT
      WRITE(*,*)'                 "i" type ',NPMTI,' (',NPMIF,NPMIZ,')'
      WRITE(*,*)'                 "o" type ',NPMTO,' (',NPMOF,NPMOZ,')'
      WRITE(*,*)'                 "u" type ',NPMTC,' (',NPMCR,NPMCZ,')'
      WRITE(*,*)'                 "d" type ',NPMTC,' (',NPMCR,NPMCZ,')'
      WRITE(*,*)'                 "l" type ',NPMTE,' (',NPMER,NPMEF,')'
      WRITE(*,*)'                 "r" type ',NPMTE,' (',NPMER,NPMEF,')'

      WRITE(*,*)'PMTs density factors "i"       type ',PMTID
      WRITE(*,*)'                     "o"       type ',PMTOD
      WRITE(*,*)'                     "c = u,d" type ',PMTCD
      WRITE(*,*)'                     "e = l,r" type ',PMTED

      WRITE(*,*)' PDZI,PDRFI = ',PDZI,PDRFI
      WRITE(*,*)' PDZO,PDRFO = ',PDZO,PDRFO
      WRITE(*,*)' PDR, PDF   = ',PDR,PDF

C MEMO
C     PDZI  ! PMT spacing in Z on the inner surface
C     PDRFI ! PMT spacing in r*dfi on the inner surface 
C     PDZO  ! PMT spacing in Z on the outer surface 
C     PDRFO ! PMT spacing in r*dfi on the outer surface 
C     PDR   ! PMT spacing in r on the side surfaces 
C     PDF   ! PMT spacing in fi on the end cups 

C Glossary: NPM(K)F   n. of PMTs on the (k) face in fi(phi) dimension
C           NPM(K)Z   n. of PMTs on the (k) face in    z    dimension    
C           NPM(K)R   n. of PMTs on the (k) face in radial  dimension
C
C              (K) = I for the inner face (x><0?)
C              (K) = O for the outer face (x><0?)
C              (K) = C for the cover face(s) up/down (y><0?)
C              (K) = E for the  end  face(s) left/right (z><0?)

      DANGI = PDRFI/LXRIN/FAC
      DANGO = PDRFO/LXREX/FAC

      KI = 0
      KO = 0
      KU = 0
      KD = 0
      KL = 0
      KR = 0
      KK = 0

C Inner surface
      WRITE(*,*)' Inner Surface'
      DO IZ = 1,NPMIZ
         ANG = - LXFIT/2.- DANGI/2.
         Z = - LXZTO/2. + FLOAT(IZ - 1)*PDZI + PDZI/2.
         DO I = 1,NPMIF
            KI = KI + 1
            ANG = (ANG + DANGI)
            KK = KK + 1
            PMTPHI(KK) = ANG + 180.
            AA = ANG*FAC
            XIPMT(KI) = LXRIN*COS(AA)
            YIPMT(KI) = LXRIN*SIN(AA)
            ZIPMT(KI) = Z
C           WRITE(*,1000) IZ,I,KI,AA,XIPMT(KI),YIPMT(KI),ZIPMT(KI)
         ENDDO
      ENDDO

C Outer Surface
      WRITE(*,*)' Outer Surface'
      DO IZ = 1,NPMOZ
         ANG = - LXFIT/2.- DANGO/2.
         Z = - LXZTO/2. + FLOAT(IZ - 1)*PDZO + PDZO/2.
         DO I = 1,NPMOF
            KO = KO + 1
            ANG = (ANG + DANGO)
            KK = KK + 1
            PMTPHI(KK) = ANG
            AA = ANG*FAC
            XOPMT(KO) = LXREX*COS(AA)
            YOPMT(KO) = LXREX*SIN(AA)
            ZOPMT(KO) = Z
C           WRITE(*,1000) IZ,I,KO,AA,XOPMT(KO),YOPMT(KO),ZOPMT(KO)
         ENDDO
      ENDDO

C Cover Up
      WRITE(*,*)' Cover Up'
      DO IZ = 1,NPMCZ
         Z = - LXZTO/2. + FLOAT(IZ - 1)*PDZO + PDZO/2.
         ANG = LXFIT/2.
         AA = ANG*FAC
         DO I = 1,NPMCR
            KU = KU + 1
            KK = KK + 1
            PMTPHI(KK)=ANG+90.
            R = LXRIN + FLOAT(I - 1)*PDR + PDR/2.
            XUPMT(KU) = R*COS(AA)
            YUPMT(KU) = R*SIN(AA)
            ZUPMT(KU) = Z
C           WRITE(*,1000) IZ,I,KU,AA,XUPMT(KU),YUPMT(KU),ZUPMT(KU)
         ENDDO

C Cover Down
         ANG = - LXFIT/2.
         AA = ANG*FAC
         DO I = 1,NPMCR
            KD = KD + 1
            KK = KK + 1
            PMTPHI(KK) = ANG - 90.
            R = LXRIN + FLOAT(I - 1)*PDR + PDR/2.
            XDPMT(KD) = R*COS(AA)
            YDPMT(KD) = R*SIN(AA)
            ZDPMT(KD) = Z
C           WRITE(*,1000) IZ,I,KD,AA,XDPMT(KD),YDPMT(KD),ZDPMT(KD)
         ENDDO         
      ENDDO                     !on z

C End Left
      DO IR = 1,NPMER
         R = LXRIN + FLOAT(IR - 1)*PDR + PDR/2.         
         WRITE(*,*)' End Left'
         ANG = -LXFIT/2. - PDF/2.
         DO I = 1,NPMEF
            KL = KL + 1
            ANG = (ANG + PDF)
            KK = KK + 1
            PMTPHI(KK) = ANG
            AA = ANG*FAC
            XLPMT(KL) = R*COS(AA)
            YLPMT(KL) = R*SIN(AA)
            ZLPMT(KL) = -LXZTO/2.
C           WRITE(*,1000) IR,I,KL,AA,R,ANG,ZLPMT(KL)
         ENDDO
         
C End Right
         WRITE(*,*)' End Right'
         ANG = - LXFIT/2. - PDF/2.
         DO I = 1,NPMEF
            KR = KR + 1
            ANG = (ANG + PDF)
            KK = KK + 1
            PMTPHI(KK) = ANG
            AA = ANG*FAC
            XRPMT(KR) = R*COS(AA)
            YRPMT(KR) = R*SIN(AA)
            ZRPMT(KR) = LXZTO/2.
C           WRITE(*,1000) IR,I,KR,AA,R,ANG,ZRPMT(KR)
         ENDDO         
      ENDDO

 1000 FORMAT(1X,3I5,3X,4F10.2)
      NNTT = KI + KO + KU + KD + KL + KR

      WRITE(*,*)'LIXPMT: total(posiz.) n. of PMTs ',NNTT
      WRITE(*,*)'                        "i" type ',KI 
      WRITE(*,*)'                        "o" type ',KO 
      WRITE(*,*)'                        "u" type ',KU 
      WRITE(*,*)'                        "d" type ',KD 
      WRITE(*,*)'                        "l" type ',KL 
      WRITE(*,*)'                        "r" type ',KR

      IF (KK.NE.NPMTT) STOP ' KK.ne.NPMT ! '
      IF (NNTT.NE.NPMTT) STOP ' NNTT.ne.NPMT ! '

      RETURN
      END

C ============================================================= C

      SUBROUTINE LIXDET

C ============================================================= C
C Define sensitive detectors in calorimeter.                    C 
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"geant321/gctmed.inc"
#include"geant321/gcvolu.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"

      CHARACTER*4 DETHITS(5)
      DATA        DETHITS/'X   ','Y   ','Z   ','E   ','ELOS'/
      INTEGER DETBITS(5)
      DATA    DETBITS/5*18/
      REAL    DETORI(5),DETFACT(5)
      DATA    DETORI/ 125., 125., 125.,   0.,    0./
      DATA    DETFACT/1000.,1000.,1000.,1000.,1000./

C Variables for hit definition on calorimeter.
      CHARACTER*4 LIXPOS(1)
      DATA LIXPOS/'LIXE'/
      INTEGER NBILIX(2),ISLIXE
      DATA NBILIX,ISLIXE/10,10,11/

      CALL GSDET('XLIQ','XLIQ',1,LIXPOS,NBILIX,ISLIXE,100,100,ISET,IDET)
      CALL GSDETH('XLIQ','XLIQ',5,DETHITS,DETBITS,DETORI,DETFACT)

      RETURN
      END

C ================================================================= C

      SUBROUTINE LIXFHI

C ================================================================= C
C Fetch hits on LiXe calorimeter.                                   C
C ================================================================= C

      IMPLICIT NONE

      RETURN
      END
 
C ================================================================= C

      SUBROUTINE LIXHIT

C ================================================================= C
C Hit loading on LiXe calorimeter.                                  C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#undef CERNLIB_GEANT321_GCKINE_INC
#undef CERNLIB_GEANT321_GTKINE_INC
#include"geant321/gckine.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#undef CERNLIB_GEANT321_GCVOLU_INC
#undef CERNLIB_GEANT321_GTVOLU_INC
#include"geant321/gcvolu.inc"
#include"uniinc/meglix.inc"

      REAL HITS(5)
      INTEGER NCELL(1),IHIT
      DATA HITS/5*0./
      DATA NCELL/1/

C HITS(5): energy release.
      HITS(5) = DESTEP*1000.
      
C HITS(1:3): position of the energy deposit. 
      HITS(1) = VECT(1)
      HITS(2) = VECT(2)
      HITS(3) = VECT(3)

C HITS(4): present energy of the particle.
      HITS(4) = GEKIN 

C The hit is defined when the particle (typically a photon) 
C releases some energy in the calorimeter.
      IF (HITS(5).GT.0) THEN
        CALL GSAHIT(ISET,IDET,ISTAK,NCELL,HITS,IHIT)

C DEBUG.
        IF (IDEBUG.NE.0) THEN
          WRITE(*,*)' ISET,IDET,ISTAK,IHIT = ',ISET,IDET,ISTAK,IHIT
          WRITE(*,*)' HITS = ',HITS
        ENDIF
        HITS(5) = 0.

      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE LIXCAR

C ============================================================= C
C Read standard cards 
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meglix.inc"

C Read LiXe calorimeter cards.
      OPEN(5,FILE = 'carlix.dat', STATUS = 'old')
      READ(5,*) LIATTE 
      READ(5,*) LXATTE
      READ(5,*) LISCAT
      READ(5,*) LXRAYL
      READ(5,*) LIPMTS 
      CLOSE (5)

      RETURN
      END

C ============================================================= C

      SUBROUTINE LIXSCI(DESTEPM)

C ============================================================= C
C Scintillation photons in LiXe.                                C
C ============================================================= C

      IMPLICIT NONE

#include"geant321/gckine.inc"
#include"geant321/gcking.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#include"uniinc/meglix.inc"

      REAL DESTEPM,AVE1
      INTEGER NPHE,NEF,I
      DATA NEF/0/
      REAL XX(6),T0,ZOUT(6),TOUT
      INTEGER ALIVE

      NPHE = DESTEPM*LXPMTQE*LXLYIELD
      NEF = NEF + NPHE

      DO I = 1,6
         XX(I) = VECT(I)
      ENDDO

      IF (NPHE.LT.1) THEN                                       !---->
         RETURN
      ELSE
         
C Generate and track photoelectrons.
         DO I = 1,NPHE
            CALL LXPHO1(XX,TOFG,ZOUT,TOUT,ALIVE)

C Compute the PMT charge.
            IF (ALIVE.GT.0) CALL LIXCHA(ZOUT,TOUT)
         ENDDO

      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE LXPHO1(ZZ,T0,ZOUT,TOUT,ALIVE)

C ============================================================= C
C Tracking of scintillation photons inside the LIXE.            C
C Variables:                                                    C
C     ZZ(1..6)   In input :  Position and direction cosines of  C
C                            energy deposit. Z(4..6) unused.    C
C     T0         In input :  Time of the energy deposit.        C
C     ZOUT(1..6) In output:  Position of the exit point and     C
C                            direction cosines of the           C
C                            exiting photon.                    C
C     TOUT       In output : Time at which the photon           C
C                            exits the LIXE.                    C
C     ALIVE      In output : 0  if the photon has been absorbed C
C                           >=1 if the photon is still alive or C
C                               hit a surface                   C
C                           -1  error condition (DIST > 10 m)   C
C ============================================================= C 

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTFLAG_INC
#undef CERNLIB_GEANT321_GCFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
C#include"geant321/gconsp.inc"
#include"uniinc/meglix.inc"

      REAL ZOUT(6), TOUT
      INTEGER ALIVE
      REAL XX(6), T0, YY(3), ZZ(6), TSCI, TPRO, TIME
      REAL RANDM(3), COST, FI, DUMMY, XC(3), DD, PROB, RNDM1
      REAL DIST, DRAY, DATT, YAPPO, APPO
      REAL RADI,PHI,PHIRAD, PATH, DISTT, PHILL,THELL,HRNDM1
      REAL THPRIM, PHPRIM, THZERO, PHZERO, PROJEC
      INTEGER I, K, L, FACE, NFACE, NSCAT
      REAL P(5),SNEXT,SNXT,SAFE,ATT
      INTEGER IACT,IFL

C Reset of parameters
      DISTT  = 0.
      NSCAT  = 0.
      DRAY   = 10000000.
      DATT   = 10000000.
      TIME   = 0.
      ALIVE  = 1
 
C XX(1..3) position of the photon
      DO I = 1,3
         XX(I)       = ZZ(I)
         XC(I)       = 0.
         ZOUT(I)     = 0.
         ZOUT(I + 3) = 0.
      ENDDO

 1    CONTINUE
     
C In which direction will the photon go?
      CALL RN3DIM(XX(4),XX(5),XX(6),1.)

 2    CONTINUE

      CALL GNEXT(XX,DIST,DUMMY)
      IF (DIST.GT.1000.) THEN
         ALIVE = -1
         GOTO 99
      ENDIF

C Extracts three random numbres
      CALL GRNDM(RANDM,3)

C Computes the distance to the next Rayleigh scattering 
      IF (LISCAT.EQ.1) THEN 
         DRAY = - LXRAYL*LOG(RANDM(1))
      ENDIF

C Computes the distance to the next absorption point 
      IF (LIATTE.EQ.1) THEN
         DATT = - LXATTE*LOG(RANDM(2))
      ENDIF

C If absorbed         => GOTO 99
      IF ((DATT.LE.DRAY).AND.(DATT.LE.DIST)) THEN
         ALIVE = 0
         GOTO 99
      ENDIF

C If hits the surface => GOTO 3
      IF ((DIST.LE.DRAY).AND.(DIST.LE.DATT)) THEN
         DISTT = DISTT + DIST
         GOTO 3
      ENDIF

C It Rayleigh-scattered, then!
      NSCAT = NSCAT + 1
      ALIVE = ALIVE + 1

C Where does the photon go?
      XX(1)  = XX(1) + DRAY*XX(4)
      XX(2)  = XX(2) + DRAY*XX(5)
      XX(3)  = XX(3) + DRAY*XX(6)
      DISTT  = DISTT + DRAY     ! Used for time calculation

C What is its new direction? 
C PHI is uniform 
      CALL GRNDM(RNDM1,1)
      PHILL = TWOPI*RNDM1

C Theta has its own distribution following 1 + cos^2(Theta)
 17   CONTINUE
      CALL GRNDM(THELL,1)
      THELL = PI*THELL
      CALL GRNDM(YAPPO,1)
      YAPPO = 2.*YAPPO         
      IF ((1 + COS(THELL)**2).LT.YAPPO) GOTO 17

C Transform the angle with respect to the original direction:
C YY(1..3) are the new photon direction cosines in ref. frame where z
C is the direction of the old photon
      YY(1) = SIN(THELL)*COS(PHILL)
      YY(2) = SIN(THELL)*SIN(PHILL)
      YY(3) = COS(THELL)

C I have to rotate YY(1..3) with z-axis along THZERO-PHZERO
C usage: CALL GDROT ( V*, COSTH, SINTH, COSPH, SINPH)
      PROJEC = SQRT(XX(4)*XX(4) + XX(5)*XX(5)) ! vector on x-y plane 
      CALL GDROT (YY,XX(6),PROJEC,XX(4)/PROJEC,XX(5)/PROJEC)

C These are the new direction cosines
      DO I = 1,3
         XX(I + 3) = YY(I)
      ENDDO
      
      GOTO 2
     
C The photon is exiting the calorimeter
 3    CONTINUE
      
C Photons arrival time on PMT
      TSCI  = - LXTAU1*LOG(RANDM(3))      ! LiXe emission time (ns)
      TPRO  = 1.E+9*LXREFIND*DISTT/CLIGHT ! photon propagation (ns)
      TIME  = 1.E+9*T0 + TSCI + TPRO      ! arrival time       (ns)

C Exit point coordinates
      DO I = 1,3
         XC(I) = XX(I) + XX(I + 3)*DIST
      ENDDO

 99   CONTINUE

      TOUT    = TIME
      ZOUT(1) = XC(1)
      ZOUT(2) = XC(2)
      ZOUT(3) = XC(3)
      ZOUT(4) = XX(4)
      ZOUT(5) = XX(5)
      ZOUT(6) = XX(6)

      RETURN
      END

C ============================================================= C

      SUBROUTINE LIXCHA(XC,TIME)

C ============================================================= C
C Given the exit point of the scintillation photon, it finds    C
C the PMT hit (if any) by this photon and increases its charge. C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTFLAG_INC
#undef CERNLIB_GEANT321_GCFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/meglix.inc"

      INTEGER FACE,NFACE,I,K,L,II
      REAL RADI,PHI,PHIRAD
      REAL EPSILON,EPSILON2
      DATA EPSILON,EPSILON2/0.1,0.5/
      REAL XC(6),TIME,DD
      INTEGER NHITCO,OLDEVENT
      DATA OLDEVENT/0/

C Find the Liquid Xenon surface.
      RADI = SQRT(XC(1)*XC(1) + XC(2)*XC(2))
      PHI  = ATAN(XC(2)/XC(1))*360./TWOPI
      PHIRAD = (PHI + 0.5*LXFIT)*TWOPI/360.

      FACE = 0
      NFACE = 0
      IF (OLDEVENT.NE.IEVENT) NHITCO = 0

      IF (ABS (RADI - LXRIN).LT.EPSILON) THEN
         FACE = 1
         NFACE = NFACE + 1
      ENDIF

      IF (ABS(RADI - LXREX).LT.EPSILON) THEN
         FACE = 2
         NFACE = NFACE + 1
      ENDIF

      IF (ABS(PHI - 0.5*LXFIT).LT.EPSILON2) THEN
         FACE = 3
         NFACE = NFACE + 1
      ENDIF

      IF (ABS(PHI + 0.5*LXFIT).LT.EPSILON2) THEN
         FACE = 4
         NFACE = NFACE + 1
      ENDIF

      IF (ABS(XC(3) + 0.5*LXZTO).LT.EPSILON) THEN
         FACE = 5
         NFACE = NFACE + 1
      ENDIF

      IF (ABS(XC(3) - 0.5*LXZTO).LT.EPSILON) THEN
         FACE = 6
         NFACE = NFACE + 1
      ENDIF

C Rejects hits on calorimeter corners and STOP if no surface is found.
      IF (NFACE.NE.1) THEN
         IF (NFACE.EQ.0) THEN 
            WRITE(*,*) ' EVENT              ',IEVENT
            WRITE(*,*) ' XC,RADI,PHI = ',(XC(II),II = 1,3),RADI,PHI
            WRITE(*,*) ' LIXCHA: face not found ! '   
C           STOP' Abnormal End ' 
            GOTO 99
         ENDIF
         IF (NFACE.GT.1) THEN
            NHITCO = NHITCO + 1
            IF (NHITCO.LE.100) THEN
               WRITE(*,*) ' LIXCHA: hit on corner '
               WRITE(*,*) ' EVENT              ',IEVENT
               WRITE(*,*) ' XC,RADI,PHI = ',(XC(II),II = 1,3),RADI,PHI
            ELSE IF (NHITCO.EQ.101) THEN
               WRITE(*,*) ' LIXCHA: more than 100 hits on corner '
               WRITE(*,*) ' EVENT              ',IEVENT
            ELSE
               CONTINUE
            ENDIF
         ENDIF
         GOTO 99
      ENDIF

 1000 FORMAT(' FACE,I,K,L,R,FI,Z ',3I3,I5,2X,3F7.1)
 1001 FORMAT(' XC = ',3F10.2)

C Inner face.
      IF (FACE.EQ.1) THEN
         I = 1. + LXRIN*PHIRAD/PDRFI
         K = 1. + (XC(3) + LXZTO/2.)/PDZI
         IF (I.EQ.NPMIF + 1) I = I - 1
         IF (K.EQ.NPMIZ + 1) K = K - 1
         L = NPMIF*(K - 1) + I

         IF (L.GT.NPMTI) THEN
            WRITE(*,*)' EVENT ',IEVENT,' WRONG I,K,L INDEXES'
            WRITE(*,1000) FACE,I,K,L,RADI,PHI,XC(3)
            WRITE(*,1001) (XC(II),II = 1,3)
            GOTO 99
         ENDIF

         DD = SQRT((XC(1) - XITPMT(L))**2 + (XC(2) - YITPMT(L))**2 +
     +             (XC(3) - ZITPMT(L))**2)
         IF (DD.LE.LXPMR) THEN
            PMTICH(L) = PMTICH(L) + 1.

C Time must be saved for the 1st photon only.
            IF (PMTITI(L).EQ.0.) PMTITI(L) = TIME
         ENDIF

C Outer face.
      ELSE IF (FACE.EQ.2) THEN
         I = 1. + LXREX*PHIRAD/PDRFO
         K = 1. + (XC(3) + LXZTO/2.)/PDZO
         IF (I.EQ.NPMOF + 1) I = I - 1
         IF (K.EQ.NPMOZ + 1) K = K - 1
         L = NPMOF*(K - 1) + I

         IF (L.GT.NPMTO) THEN
            WRITE(*,*)' EVENT ',IEVENT,' WRONG I,K,L INDEXES '
            WRITE(*,1000) FACE,I,K,L,RADI,PHI,XC(3)
            WRITE(*,1001) (XC(II),II = 1,3)
            GOTO 99
         ENDIF

         DD = SQRT((XC(1) - XOTPMT(L))**2 + (XC(2) - YOTPMT(L))**2 +
     +             (XC(3) - ZOTPMT(L))**2)
         IF (DD.LE.LXPMR) THEN
            PMTOCH(L) = PMTOCH(L) + 1.

C Time must be saved for the 1st photon only.
            IF (PMTOTI(L).EQ.0.) PMTOTI(L) = TIME
         ENDIF

C Up cover.
      ELSE IF (FACE.EQ.3) THEN
         I = 1. + (RADI - LXRIN)/PDR
         K = 1. + (XC(3) + LXZTO/2.)/PDZO
         IF (I.EQ.NPMCR + 1) I = I - 1
         IF (K.EQ.NPMCZ + 1) K = K - 1
         L = NPMCR*(K - 1) + I

         IF (L.GT.NPMTC) THEN
            WRITE(*,*)' EVENT ',IEVENT,' WRONG I,K,L INDEXES '
            WRITE(*,1000) FACE,I,K,L,RADI,PHI,XC(3)
            WRITE(*,1001) (XC(II),II = 1,3)
            GOTO 99
         ENDIF

         DD = SQRT((XC(1) - XUTPMT(L))**2 + (XC(2) - YUTPMT(L))**2 +
     +             (XC(3) - ZUTPMT(L))**2)
         IF (DD.LE.LXPMR) THEN
            PMTUCH(L) = PMTUCH(L) + 1.

C Time must be saved for the 1st photon only.
            IF (PMTUTI(L).EQ.0.) PMTUTI(L) = TIME
         ENDIF

C Down cover.
      ELSE IF (FACE.EQ.4) THEN
         I = 1. + (RADI - LXRIN)/PDR
         K = 1. + (XC(3) + LXZTO/2.)/PDZO
         IF (I.EQ.NPMCR + 1) I = I - 1
         IF (K.EQ.NPMCZ + 1) K = K - 1
         L = NPMCR*(K - 1) + I

         IF (L.GT.NPMTC) THEN
            WRITE(*,*)' EVENT ',IEVENT,' WRONG I,K,L INDEXES '
            WRITE(*,1000) FACE,I,K,L,RADI,PHI,XC(3)
            WRITE(*,1001) (XC(II),II = 1,3)
            GOTO 99
         ENDIF

         DD = SQRT((XC(1) - XDTPMT(L))**2 + (XC(2) - YDTPMT(L))**2 +
     +             (XC(3) - ZDTPMT(L))**2)
         IF (DD.LE.LXPMR) THEN
            PMTDCH(L) = PMTDCH(L) + 1.

C Time must be saved for the 1st photon only.
            IF (PMTDTI(L).EQ.0.) PMTDTI(L) = TIME
         ENDIF

C End left.
      ELSE IF (FACE.EQ.5) THEN
         I = 1. + (PHI + LXFIT/2.)/PDF
         K = 1. + (RADI - LXRIN)/PDR
         IF (I.EQ.NPMEF + 1) I = I - 1
         IF (K.EQ.NPMER + 1) K = K - 1
         L = NPMEF*(K - 1) + I

         IF (L.GT.NPMTE) THEN
            WRITE(*,*)' EVENT ',IEVENT,' WRONG I,K,L INDEXES '
            WRITE(*,1000) FACE,I,K,L,RADI,PHI,XC(3)
            WRITE(*,1001) (XC(II),II = 1,3)
            GOTO 99
         ENDIF

         DD = SQRT((XC(1) - XLTPMT(L))**2 + (XC(2) - YLTPMT(L))**2 +
     +             (XC(3) - ZLTPMT(L))**2)
         IF (DD.LE.LXPMR) THEN
            PMTLCH(L) = PMTLCH(L) + 1.

C Time must be saved for the 1st photon only.
            IF (PMTLTI(L).EQ.0.) PMTLTI(L) = TIME
         ENDIF

C End right.
      ELSE IF (FACE.EQ.6) THEN
         I = 1. + (PHI + LXFIT/2.)/PDF
         K = 1. + (RADI - LXRIN)/PDR
         IF (I.EQ.NPMEF + 1) I = I - 1
         IF (K.EQ.NPMER + 1) K = K - 1
         L = NPMEF*(K - 1) + I

         IF (L.GT.NPMTE) THEN
            WRITE(*,*)' EVENT ',IEVENT,' WRONG I,K,L INDEXES '
            WRITE(*,1000) FACE,I,K,L,RADI,PHI,XC(3)
            WRITE(*,1001) (XC(II),II = 1,3)
            GOTO 99
         ENDIF

         DD = SQRT((XC(1) - XRTPMT(L))**2 + (XC(2) - YRTPMT(L))**2 +
     +             (XC(3) - ZRTPMT(L))**2)
         IF (DD.LE.LXPMR) THEN
            PMTRCH(L) = PMTRCH(L) + 1.

C Time must be saved for the 1st photon only.
            IF (PMTRTI(L).EQ.0.) PMTRTI(L) = TIME
         ENDIF

      ENDIF

C LiXe pulse shape.
C      CALL LIXEPU(FACE,L,TIME)

 99   CONTINUE
      OLDEVENT = IEVENT

      RETURN
      END

C ============================================================= C

      SUBROUTINE LXPHAR(ZZ,T0,ZOUT,TOUT,ALIVE,ATTNEW,RAYNEW)

C ============================================================= C
C This subroutine tracks scintillation photons inside the LIXE. C
C ============================================================= C

C ============================================================= C
C Variables:                                                    C
C ZZ(1..6)   In input :  Position and direction cosines of      C
C                        energy deposit. Z(4..6) unused         C
C T0         In input :  Time of the energy deposit.            C
C ZOUT(1..6) In output:  Position of the exit point and         C
C                        direction cosines of the               C
C                        exiting photon.                        C
C TOUT       In output : Time at which the photon               C 
C                        exits the LIXE.                        C
C ALIVE      In output : 0  if the photon has been absorbed     C
C                        1  if the photon is still alive        C
C ATTNEW     In input  : Absorption length (in cm)              C
C RAYNEW               : Rayleigh scattering length (in cm)     C
C ============================================================= C     

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTFLAG_INC
#undef CERNLIB_GEANT321_GCFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
C#include"geant321/gconsp.inc"
#include"uniinc/meglix.inc"

      REAL ZOUT(6), TOUT
      INTEGER ALIVE
      REAL XX(6), T0, YY(3), ZZ(6), TSCI, TPRO, TIME
      REAL RANDM(3), COST, FI, DUMMY, XC(3), DD, PROB, RNDM1
      REAL DIST, DRAY, DATT, YAPPO, APPO, RAYNEW, ATTNEW
      REAL RADI,PHI,PHIRAD, PATH, DISTT, PHILL,THELL,HRNDM1
      REAL THPRIM, PHPRIM, THZERO, PHZERO, PROJEC
      INTEGER I, K, L, FACE, NFACE, NSCAT
      REAL P(5),SNEXT,SNXT,SAFE,ATT
      INTEGER IACT,IFL

C Reset of parameters
      DISTT  = 0.
      NSCAT  = 0.
      DRAY   = 10000000.
      DATT   = 10000000.
      TIME   = 0.
      ALIVE  = 1
 
C XX(1..3) position of the photon
      DO I = 1,3
         XX(I)       = ZZ(I)
         XC(I)       = 0.
         ZOUT(I)     = 0.
         ZOUT(I + 3) = 0.
      ENDDO

 1    CONTINUE
     
C In which direction will the photon go?
      CALL RN3DIM( XX(4), XX(5), XX(6), 1.)

 2    CONTINUE

      CALL GNEXT(XX, DIST, DUMMY)
      IF (DIST.GT.1000.) THEN
         ALIVE = -1
         GOTO 99
      ENDIF

C Extracts three random numbres
      CALL GRNDM(RANDM,3)

C Computes the distance to the next Raylaigh scattering 
      IF (LISCAT.EQ.1) THEN 
         DRAY = - RAYNEW*LOG(RANDM(1))
      ENDIF

C Computes the distance to the next absorption point 
      IF (LIATTE.EQ.1) THEN
         DATT = - ATTNEW*LOG(RANDM(2))
      ENDIF

C If absorbed         => GOTO 99
      IF ((DATT.LE.DRAY).AND.(DATT.LE.DIST)) THEN
         ALIVE = 0
         GOTO 99
      ENDIF

C If hits the surface => GOTO 3
      IF ((DIST.LE.DRAY).AND.(DIST.LE.DATT)) THEN
         DISTT = DISTT + DIST
         GOTO 3
      ENDIF

C It Rayleigh-scattered, then!
      NSCAT = NSCAT + 1
      ALIVE = ALIVE + 1

C Where does the photon go?
      XX(1)  = XX(1) + DRAY*XX(4)
      XX(2)  = XX(2) + DRAY*XX(5)
      XX(3)  = XX(3) + DRAY*XX(6)
      DISTT  = DISTT + DRAY     ! Used for time calculation

C What is its new direction? 
C PHI is random 
      CALL GRNDM(RNDM1,1)
      PHILL=TWOPI*RNDM1

C Theta has its own distribution following 1+cos^2(Theta)
 17   CONTINUE
      CALL GRNDM(THELL,1)
      THELL = 3.14159265*THELL
      CALL GRNDM(YAPPO,1)
      YAPPO = 2.*YAPPO         
      IF ((1 + COS(THELL)**2).LT.YAPPO) GOTO 17

C Transform the angle with respect to the original direction:
C YY(1..3) are the new photon directory cosines in ref. frame where z
C is the direction of the old photon
      YY(1) = SIN(THELL)*COS(PHILL)
      YY(2) = SIN(THELL)*SIN(PHILL)
      YY(3) = COS(THELL)

C I have to rotate YY(1..3) with z-axis along THZERO-PHZERO
C usage: CALL GDROT (V*, COSTH, SINTH, COSPH, SINPH)
      PROJEC = SQRT(XX(4)*XX(4) + XX(5)*XX(5)) ! vector on x-y plane 
      CALL GDROT (YY,XX(6),PROJEC,XX(4)/PROJEC,XX(5)/PROJEC)

C These are the new direction cosines
      DO I = 1,3
         XX(I + 3) = YY(I)
      ENDDO
      
      GOTO 2
     
C The photon is exiting the calorimeter
 3    CONTINUE
      
C Photons arrival time on PMT
      TSCI  = - LXTAU1 * LOG(RANDM(3))          ! LiXe emission time (ns)
      TPRO  = 1.E+9 * LXREFIND * DISTT / CLIGHT ! photon propagation (ns)
      TIME  = 1.E+9 * T0 + TSCI + TPRO          ! arrival time       (ns)

C Exit point coordinates
      DO I = 1,3
         XC(I) = XX(I) + XX(I + 3)*DIST
      ENDDO

 99   CONTINUE

      TOUT    = TIME
      ZOUT(1) = XC(1)
      ZOUT(2) = XC(2)
      ZOUT(3) = XC(3)
      ZOUT(4) = XX(4)
      ZOUT(5) = XX(5)
      ZOUT(6) = XX(6)

      RETURN
      END







