C ============================================================= C
C This package contains some general purpose routines, which    C
C includes light transport and collection, detector drawing,    C 
C data writing and reading etc.                                 C
C Part of this package is still under development.              C
C ============================================================= C

C ============================================================= C
C Light collection.                                             C
C ============================================================= C

C ============================================================= C

      SUBROUTINE LXPHOT(ZZ,T0,ZOUT,TOUT,ALIVE)
 
C ============================================================= C              
C     *** DEVELOPMENT VERSION  *** DO NOT USE *** ** *          C
C ============================================================= C

C ============================================================= C        
C This subroutine tracks scintillation photons inside the LIXE. C
C Variables:                                                    C
C     ZZ(1..6)   In input :  Position and direction cosines     C
C                            of energy deposit. Z(4..6) unused  C
C     T0         In input :  Time of the energy deposit.        C
C     ZOUT(1..6) In output:  Position of the exit point and     C 
C                            direction cosines of the           C
C                            exiting photon.                    C
C     TOUT       In output : Time at which the photon exits     C 
C                            the LIXE.                          C
C     ALIVE      In output : 0  if the photon was absorbed      C
C                            1  if the photon is still alive    C
C ============================================================= C 

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTFLAG_INC
#undef CERNLIB_GEANT321_GCFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/meglix.inc"

      REAL ZOUT(6), TOUT
      INTEGER ALIVE
      REAL XX(6),T0,YY(3),ZZ(6),TSCI,TPRO,TIME
      REAL COST,FI,DUMMY,XC(3),DD,PROB,RNDM1
      REAL DIST,DRAY,DATT,YAPPO,APPO
      REAL RADI,PHI,PHIRAD,PATH,DISTT,PHILL,THELL,HRNDM1
      REAL THPRIM,PHPRIM,THZERO,PHZERO,PROJEC
      INTEGER I,K,L,FACE,NFACE,NSCAT
      REAL P(5),SNEXT,SNXT,SAFE,ATT
      INTEGER IACT,IFL

C Reset of parameters.
      DISTT  = 0.
C      NSCAT  = 0.
      DRAY   = 10000000.
      DATT   = 10000000.
      TIME   = 0.
      ALIVE  = 1
 
C XX(1..3) position of the photon.
      DO I = 1,3
         XX(I)       = ZZ(I)
         XC(I)       = 0.
         ZOUT(I)     = 0.
         ZOUT(I + 3) = 0.
      ENDDO
 
 1    CONTINUE
     
C In which direction will the photon go ?
      CALL RN3DIM( XX(4), XX(5), XX(6), 1.)

 2    CONTINUE

C Computes the distance to the volume boundary
C First of all: fetches LIXE parameter in P(5)
C      P(1) =  LXRIN
C      P(2) =  LXREX
C      P(3) =  0.5*LXZTO
C      P(4) = -0.5*LXFIT+180.
C      P(5) =  0.5*LXFIT+180.
C      CALL GNTUBE(XX, P, 2, 2, 10000., DIST, DUMMY)
C.    ******************************************************************
C.    *                                                                *
C.    *      Compute distance to intersection with boundary surface of *
C     *      volume TUBE or TUBS, from point X(1),X(2),X(3) INSIDE     *
C     *      the volume along track with direction cosines X(4),X(5),  *
C     *      X(6)                                                      *
C.    *      P     (input)  : volume parameters                        *
C.    *      IACT  (input)  : action flag                              *
C.    *         = 0  Compute SAFE only                                 *
C.    *         = 1  Compute SAFE, compute SNXT only if SAFE.LT.SNEXT  *
C.    *         = 2  Compute both SAFE and SNXT                        *
C.    *         = 3  Compute SNXT only                                 *
C.    *      IFL   (input)  : 1 for TUBE, 2 for PHI segmented TUBE     *
C.    *      SNEXT (input)  : see IACT = 1                             *
C.    *      SNXT  (output) : distance to volume boundary along track  *
C.    *      SAFE  (output) : not larger than scalar distance to       *
C.    *                       volume boundaray                         *
C.    *      Called by : GNEXT, GNOPCO, GTNEXT                         *
C.    *                                                                *
C.    *      Authors   : Michel Maire and Rolf Nierhaus   21-JUN-1990  *
C.    *                                                                *
C.    ******************************************************************
C.    *                                                                *
C.    * 'TUBE'    is a tube. It has 3 parameters, the inside radius,   *
C.    *           the outside radius and the half length in z.         *
C.    * 'TUBS'    is a phi segment of a  tube.  It has 5 parameters,   *
C.    *           the same  3 as  'TUBE' plus  the phi  limits.  The   *
C.    *           segment  starts at  the first  limit and  includes   *
C.    *           increasing phi  value up  to the  second limit  or   *
C.    *           that plus 360 degrees.                               *
C.    *                                                                *
C.    ******************************************************************
      CALL GNEXT(XX, DIST, DUMMY)
      IF (DIST.GT.500.) THEN
         GOTO 1
      ENDIF

C Computes the distance to the next Rayleigh scattering 
      IF (LISCAT.EQ.1) THEN 
         CALL GRNDM(APPO,1)
         DRAY = - LXRAYL * LOG(APPO)
      ENDIF

C Computes the distance to the next absorption point 
      IF (LIATTE.EQ.1) THEN
         CALL GRNDM(APPO,1)
         DATT = - LXATTE * LOG(APPO)
      ENDIF

C If absorbed         => GOTO 99
      IF ((DATT.LE.DRAY).AND.(DATT.LE.DIST)) THEN
         ALIVE = 0
         GOTO 99
      ENDIF

C If hits the surface => GOTO 3
      IF ((DIST.LE.DRAY).AND.(DIST.LE.DATT)) THEN
         DISTT = DISTT + DIST
         GOTO 3
      ENDIF
 
C It Rayleigh-scattered, then!
C      NSCAT = NSCAT + 1
C
C Where does the photon go?
      XX(1)  = XX(1) + DRAY*XX(4)
      XX(2)  = XX(2) + DRAY*XX(5)
      XX(3)  = XX(3) + DRAY*XX(6)
      DISTT = DISTT + DRAY     ! Used for time calculation

C What is its new direction? 
C PHI is random 
      CALL GRNDM(RNDM1,1)
      PHILL = TWOPI*RNDM1

C Theta has its own distribution following 1 + cos^2(Theta)
 17   CONTINUE
      CALL GRNDM(RNDM1,1)
      THELL = 3.14159265*RNDM1
      CALL GRNDM(RNDM1,1)
      YAPPO = 2.*RNDM1         
      IF ((1 + COS(THELL)**2).LT.YAPPO) GOTO 17

C Transform the angle with respect to the original direction:
C YY(1..3) are the new photon direction cosines in ref. frame where z
C is the direction of the old photon
      YY(1) = SIN(THELL)*COS(PHILL)
      YY(2) = SIN(THELL)*SIN(PHILL)
      YY(3) = COS(THELL)

C I have to rotate YY(1..3) with z-axis along THZERO-PHZERO
C usage: CALL GDROT ( V*, COSTH, SINTH, COSPH, SINPH)
      PROJEC = SQRT(XX(4)*XX(4) + XX(5)*XX(5)) ! vector on x-y plane 
      CALL GDROT (YY,XX(6),PROJEC,XX(4)/PROJEC,XX(5)/PROJEC)

C These are the new direction cosines
      DO I = 1,3
         XX(I + 3) = YY(I)
      ENDDO
      
      GOTO 2
     
C The photon is exiting the calorimeter
 3    CONTINUE
      
C Photons arrival time on PMT
      CALL GRNDM(RNDM1,1)
      TSCI = - LXTAU1*LOG(1.- RNDM1)      ! LiXe emission time
      TPRO = 1.E+9*LXREFIND*DISTT/CLIGHT   ! photon propagation
      TIME = 1.E+9*T0 + TSCI + TPRO 

C Exit point coordinates
      DO I = 1,3
         XC(I) = XX(I) + XX(I + 3)*DIST
      ENDDO

 99   CONTINUE

      TOUT    = TIME
      ZOUT(1) = XC(1)
      ZOUT(2) = XC(2)
      ZOUT(3) = XC(3)
      ZOUT(4) = XX(4)
      ZOUT(5) = XX(5)
      ZOUT(6) = XX(6)

      RETURN
      END

C ============================================================= C

      SUBROUTINE LIXEPU(IFACE,IPM,TIME)

C ============================================================= C
C Liquid Xenon pulse formation. It assumes 10 ns bins and 300   C
C ns of timing depth for ADCs. The results are stored in 2-dim  C
C vectors.                                                      C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER IFACE,IPM
      INTEGER IBIN
      REAL TIME

      IBIN = 1 + INT(TIME/BINWID)
      IF (IBIN.LE.1) IBIN = 1
      IF (IBIN.GE.NBADC) IBIN = NBADC

      IF (IFACE.EQ.1) THEN
         PULI(IPM,IBIN) = PULI(IPM,IBIN) + 1
      ELSE IF (IFACE.EQ.2) THEN
         PULO(IPM,IBIN) = PULO(IPM,IBIN) + 1
      ELSE IF (IFACE.EQ.3) THEN
         PULU(IPM,IBIN) = PULU(IPM,IBIN) + 1
      ELSE IF (IFACE.EQ.4) THEN
         PULD(IPM,IBIN) = PULD(IPM,IBIN) + 1
      ELSE IF (IFACE.EQ.5) THEN
         PULL(IPM,IBIN) = PULL(IPM,IBIN) + 1
      ELSE IF (IFACE.EQ.6) THEN
         PULR(IPM,IBIN) = PULR(IPM,IBIN) + 1
      ENDIF

      RETURN
      END

C ============================================================= C
C Detector drawing.                                             C
C ============================================================= C

C ============================================================= C

      SUBROUTINE MEGDRW(XC,YC,THE,PHI,PSI)

C ============================================================= C
C Complete detector drawing. XC and YC are the coordinates of   C
C the center of the figure.                                     C
C ============================================================= C

      IMPLICIT NONE

      REAL XC,YC,XX,YY,XSIZ,YSIZ,THE,PHI,PSI

C ============================================================= C
C Compute sizes and viewing angles of the various plots.        C
C ============================================================= C

C One view only.
      IF (XC.EQ.10.AND.YC.EQ.10) THEN
         XX = 16.
         YY = 10.
         XSIZ = 0.08
         YSIZ = 0.08

C Four views: angles 0,0
      ELSE IF (XC.EQ.5.AND.YC.EQ.15.5) THEN
         XX = 8.5
         YY = 14.5
         XSIZ = 0.035
         YSIZ = 0.035

C Four views: cut view.
      ELSE IF (XC.EQ.14.AND.YC.EQ.5.5) THEN
         XX = -1000.
         YY = -1000.
         XSIZ = 0.033
         YSIZ = 0.033

C Four views: horizontal plane.
      ELSE IF (XC.EQ.15.AND.YC.EQ.15.5) THEN
         XX = -1000.
         YY = -1000.
         XSIZ = 0.035
         YSIZ = 0.035

C Four views: lateral plane.
      ELSE IF (XC.EQ.5.AND.YC.EQ.5.5) THEN
         XX = -1000.
         YY = -1000.
         XSIZ = 0.035
         YSIZ = 0.035
      ENDIF

C Draw them.
      CALL GDRAW('FRAM',THE,PHI,PSI,XC,YC,XSIZ,YSIZ)
      CALL GDXYZ(0)
      CALL GDWMN2(XX,YY)

      RETURN
      END

C ============================================================= C

      SUBROUTINE MEGHIT

C ============================================================= C
C Draw the hits.                                                C
C ============================================================= C

      IMPLICIT NONE

      CALL IGSET('TXCI',2.)
      CALL GDHITS('TOFI','TOFI',0,850,.15)
      CALL IGSET('TXCI',4.)
      CALL GDHITS('TOFO','TOFO',0,850,.15)
      CALL IGSET('TXCI',6.)
      CALL GDHITS('SCHA','SCHA',0,850,.15)
      CALL IGSET('TXCI',1.)
      CALL IGSET('TXCI',7.)
      CALL GDHITS('XLIQ','XLIQ',0,850,.05)
      CALL IGSET('TXCI',1.)

      RETURN
      END

C ============================================================= C

      SUBROUTINE HIFSET

C ============================================================= C
C Set the hidding/filling algorithm.                            C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguse.inc"

      IF (IHID.EQ.0) THEN
         RETURN

C Set the hidding/shadowing algorithm.
      ELSE IF (IHID.EQ.1) THEN
         CALL GDOPT('HIDE','ON  ')
         CALL GDOPT('SHAD','ON  ')

C Set the area style and filling option.
         IF (IFIL.EQ.1) THEN
            CALL IGSET('FASI',3.)
            CALL IGSET('FAIS',3.)
         ENDIF
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE DRASET

C ============================================================= C
C Define the drawing requests for the various portions of the   C
C detector.                                                     C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguse.inc"

C General frame.
      CALL GSATT('FRAM','SEEN',0)

C ============================================================= C
C Lixe.                                                         C
C ============================================================= C

C PMTs (they are shown only if the outside barrels are not).
      IF (IHID.EQ.0) CALL GSATT('XPMT','SEEN',0)
      CALL GSATT('XPMT','COLO',2)

C LiXe volume.
      IF (IHID.EQ.1) CALL GSATT('XLIQ','FILL',7)
      CALL GSATT('XLIQ','COLO',5)

C Plastic barrel.
      IF (IHID.EQ.1) CALL GSATT('XPLA','SEEN',0)
      CALL GSATT('XPLA','COLO',1)

C Internal aluminum barrel.
      IF (IHID.EQ.1) CALL GSATT('XALI','SEEN',0)
      CALL GSATT('XALI','COLO',4)

C Vacuum barrel.
      IF (IHID.EQ.1) CALL GSATT('XVAC','SEEN',0)
      CALL GSATT('XVAC','COLO',1)

C External aluminum barrel.
      IF (IHID.EQ.1) CALL GSATT('XALE','SEEN',0)
      CALL GSATT('XALE','COLO',4)

C Cover UP.
      IF (IHID.EQ.1) CALL GSATT('XPLU','SEEN',0)
      CALL GSATT('XPLU','COLO',1)
      IF (IHID.EQ.1) CALL GSATT('XAEU','SEEN',0)
      CALL GSATT('XAEU','COLO',4)
      IF (IHID.EQ.1) CALL GSATT('XVAU','SEEN',0)
      CALL GSATT('XVAU','COLO',1)
      IF (IHID.EQ.1) CALL GSATT('XAUU','SEEN',0)
      CALL GSATT('XAUU','COLO',4)

C Cover DOWN.
      IF (IHID.EQ.1) CALL GSATT('XPLD','SEEN',0)
      CALL GSATT('XPLD','COLO',1)
      IF (IHID.EQ.1) CALL GSATT('XAED','SEEN',0)
      CALL GSATT('XAED','COLO',4)
      IF (IHID.EQ.1) CALL GSATT('XVAD','SEEN',0)
      CALL GSATT('XVAD','COLO',1)
      IF (IHID.EQ.1) CALL GSATT('XADD','SEEN',0)
      CALL GSATT('XADD','COLO',4)

C LiXe frame.
      CALL GSATT('XFRA','SEEN',0)
      CALL GSATT('XFRA','COLO',1)

C ============================================================= C
C Spectrometer.                                                 C
C ============================================================= C

C Drift chamber.
      CALL GSATT('SCHA','COLO',1)

C Active gas layer.
      CALL GSATT('SGAS','COLO',7)

C Kapton gas tight layer.
      CALL GSATT('SKAP','COLO',3)

C G10 support.
      CALL GSATT('SG10','COLO',4)

C Spectrometer frame.
      CALL GSATT('SFRA','SEEN',0)
      CALL GSATT('SFRA','COLO',1)

C ============================================================= C
C Timing counter.                                               C
C ============================================================= C

C Timing counter: inner scintillator.
      CALL GSATT('TOFI','COLO',3)

C Timing counter: barrels.
      IF (IHID.EQ.0) CALL GSATT('TOF1','SEEN',0)
      IF (IHID.EQ.0) CALL GSATT('TOF2','SEEN',0)

C Timing counter: outer scintillator.
      CALL GSATT('TOFO','COLO',7)

C Timing counter frame.
      CALL GSATT('TFRA','SEEN',0)
      CALL GSATT('TFRA','COLO',1)

C ============================================================= C
C Target.                                                       C
C ============================================================= C

C Target.
      IF (IHID.EQ.1) CALL GSATT('TARG','FILL',7)
      CALL GSATT('TARG','COLO',2)

C ============================================================= C
C Coils.                                                        C
C ============================================================= C

C Coils.
      CALL GSATT('CSCO','COLO',1)
      CALL GSATT('CCIW','COLO',1)
      CALL GSATT('CSIW','COLO',1)
      CALL GSATT('CCOW','COLO',1)
      CALL GSATT('CCCO','COLO',1)
      CALL GSATT('CMCO','COLO',1)
      CALL GSATT('CMIW','COLO',1)
      CALL GSATT('CASC','COLO',1)

C Coil frame.
      CALL GSATT('CFRA','SEEN',0)

      RETURN
      END

C ============================================================= C
C Data writing.                                                 C
C ============================================================= C

C ============================================================= C

      SUBROUTINE MEGOUT(IDATA)

C ============================================================= C
C Unformatted output file.                                      C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#include"uniinc/megeve.inc"
#include"uniinc/meguti.inc"

C ============================================================= C
C Output file structure.                                        C
C ============================================================= C
C This structure will need some modification when the hardware  C
C setup will reach its final configuration. Presently we can    C
C not say which will be the available event information         C
C (i.e. number of chambers, readout etc.) in all details.       C
C Then, we presently store the high level information (GEANT    C
C hits and energy loss), having in mind that somewhat will be   C
C changed in the future.                                        C
C ============================================================= C
C                                                               C
C Header: IDATA      Data type: 1 = Data, 0 = MC                C
C         IEVENT     Event number                               C
C         IPOS       e+    FLAG (= 1: e+ present, 0 = not)      C
C              if IPOS = 1:                                     C          
C                 PPOS(3) e+ momentum                           C 
C                 XPOS(3) e+ emission point                     C
C         IGAM       GAMMA FLAG (= 1: gamma present, 0 = not)   C
C              if IGAM = 1:                                     C
C                 PGAM(3) photon momentum                       C 
C                 XGAM(3) photon emission point                 C
C                                                               C
C Energy loss:                                                  C
C         ELXE energy loss in Liquid Xenon                      C
C         EPTL energy loss in Plastic (PET, calorimeter)        C
C         EPMT energy loss in PMTs                              C
C         EPVT energy loss in PVT                               C
C         ENIT energy loss in Nitrogen                          C
C         EKAP energy loss in Kapton                            C
C         EHET energy loss in Helium-Ethane                     C
C         EHEL energy loss in Helium                            C
C         EG10 energy loss in G10                               C
C         EPTS energy loss in Plastic (PET, spectrometer)       C
C         EALC energy loss in Aluminum                          C
C         EVCC energy loss in Vacuum                            C
C         EPTT energy loss in PVT                               C
C                                                               C
C Gamma interaction point and average coordinates               C
C         X0,Y0,Z0      Gamma first interaction point           C 
C         CONVP(3)      Gamma conversion point (if it exists)   C
C         XAVE(3)       Gamma center of energy deposition       C
C         LONFL         Shower longitudinal fluctuation         C
C         LATFL         Shower lateral fluctuation              C
C                                                               C
C Lixe data:  (DO NOT INCLUDE TIMES AND PULSES BY NOW)          C
C         NPMTI         Number of PMTs on the I face            C
C         PMTICH(NPMTI) PMT charge     on the I face            C
C         PMTITI(NPMTI) PMT times      on the I face            C
C         NPMTO         Number of PMTs on the O face            C
C         PMTOCH(NPMTO) PMT charge     on the O face            C
C         PMTOTI(NPMTO) PMT times      on the O face            C
C         NPMTU(C)      Number of PMTs on the U face (U COVER)  C
C         PMTUCH(NPMTC) PMT charge     on the U face            C
C         PMTUTI(NPMTC) PMT times      on the U face            C
C         NPMTD(C)      Number of PMTs on the D face (D COVER)  C
C         PMTDCH(NPMTC) PMT charge     on the D face            C
C         PMTDTI(NPMTC) PMT times      on the D face            C
C         NPMTL(E)      Number of PMTs on the L face (L END     C
C         PMTLCH(NPMTE) PMT charge     on the L face            C
C         PMTLTI(NPMTE) PMT times      on the L face            C
C         NPMTR(E)      Number of PMTs on the R face (R END)    C
C         PMTRCH(NPMTE) PMT charge     on the R face            C
C         PMTRTI(NPMTE) PMT times      on the R face            C
C         PULI(NPMTI,NBADC) ADC pulse  on the I face            C
C         PULO(NPMTO,NBADC) ADC pulse  on the O face            C
C         PULU(NPMTU,NBADC) ADC pulse  on the U face            C
C         PULD(NPMTD,NBADC) ADC pulse  on the D face            C
C         PULL(NPMTL,NBADC) ADC pulse  on the L face            C
C         PULR(NPMTR,NBADC) ADC pulse  on the R face            C
C                                                               C
C Timing counter data:                                          C
C         NPHOD         Number of Timing Counter HITS           C
C         TCH(NPHOD,8)  Timing Counter HITS coordinates(1-3),   C
C                       energy release(4), time(5), angle(6),   C
C                       hit cell(7), timing counter code (8)    C
C                       (1=in, 2=out)                           C
C                                                               C
C Spectrometer data:                                            C
C         NTRK1         Number of Spectrometer HITS             C
C         SPH(NTRK1,7)  Spectrometer HITS coordinates(1-3),     C
C                       energy release(4), time(5), angle(6),   C
C                       energy loss by bremsstrahlung(7)        C
C ============================================================= C
                         
      INTEGER ITIME
      DATA ITIME/0/
      INTEGER NV,IV,I
      INTEGER IGAM,IPOS,IDATA

      IF (ITIME.EQ.0) THEN
         ITIME = 1
         OPEN(LUNOUT,FILE = 'megumc.out',FORM = 'UNFORMATTED', 
     +        STATUS = 'UNKNOWN')

C MC Data.
         IF (IDATA.EQ.0) THEN
 
C TYPE <= 9: pair events.
            IF (TYPE.LE.9) THEN
               IGAM = 1
               IPOS = 1

C 11 <= ITYPE <= 19: e+ only events.
            ELSE IF (TYPE.GE.11.AND.TYPE.LE.19) THEN
               IGAM = 0
               IPOS = 1
               
C 21 <= ITYPE <= 24: gamma only events.
            ELSE IF (TYPE.GE.21.AND.TYPE.LE.24) THEN
               IGAM = 1
               IPOS = 0

C 25 <= ITYPE <= 28: pile-up events.
            ELSE IF (TYPE.GE.25.AND.TYPE.LE.28) THEN
               IGAM = 2
               IPOS = 0

            ENDIF

         ENDIF
      ENDIF

      NV = 0

C Header.
      IV = 1
      V(IV) = FLOAT(IDATA)

      IV = IV + 1
      V(IV) = FLOAT(IEVENT)

C MC Data.
      IF (IDATA.EQ.0) THEN
         IV = IV + 1 
         V(IV) = FLOAT(IPOS)

         IF (IPOS.EQ.1) THEN
            DO I = 1,3
               V(IV + I)     = PINIT(1,I)
               V(IV + 3 + I) = VINIT(1,I)
            ENDDO
            IV = IV + 6
         ENDIF
         
         IV = IV + 1
         V(IV) = FLOAT(IGAM)
         
         IF (IGAM.EQ.1) THEN
            DO I = 1,3
               V(IV + I)     = PINIT(2,I)
               V(IV + 3 + I) = VINIT(2,I)
            ENDDO
            IV = IV + 6

         ELSE IF (IGAM.EQ.2) THEN
            DO I = 1,3
               V(IV + I)     = PINIT(1,I)
               V(IV + 3 + I) = VINIT(1,I)
            ENDDO
            IV = IV + 6

            DO I = 1,3
               V(IV + I)     = PINIT(2,I)
               V(IV + 3 + I) = VINIT(2,I)
            ENDDO
            IV = IV + 6
         ENDIF

         IV = IV + 1

C Energy loss.
         CALL ENELOS(IV)

C ATTIVARLA PER FARE NUOVI FILES
         IV = IV + 1

C Gamma interaction point.
         CALL INTPOI(IV)

      ENDIF
      IV = IV + 1

C LiXe buffer.
      CALL LIXBUF(IV)
      IV = IV + 1

C Tof buffer.
      CALL TOFBUF(IV)
      IV = IV + 1

C Spectrometer buffer.
      CALL SPEBUF(IV)

C Write all buffers.
      NV = IV
      WRITE(LUNOUT) NV,(V(IV),IV = 1,NV)      

      RETURN
      END

C ============================================================= C

      SUBROUTINE ENELOS(IV)

C ============================================================= C
C Energy loss in various materials.                             C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"
#include"uniinc/meguse.inc"

      INTEGER IV

      V(IV     ) = ENEDEP(22)
      V(IV +  1) = ENEDEP(25)
      V(IV +  2) = ENEDEP(26)
      V(IV +  3) = ENEDEP(31)
      V(IV +  4) = ENEDEP(32)
      V(IV +  5) = ENEDEP(41)
      V(IV +  6) = ENEDEP(42)
      V(IV +  7) = ENEDEP(43)
      V(IV +  8) = ENEDEP(44)
      V(IV +  9) = ENEDEP(45)
      V(IV + 10) = ENEDEP(51)
      V(IV + 11) = ENEDEP(52)
      V(IV + 12) = ENEDEP(61)

      IV = IV + 12

      RETURN
      END

C ============================================================= C

      SUBROUTINE INTPOI(IV)

C ============================================================= C
C Gamma interaction and conversion point and center of energy   C
C deposition.                                                   C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"
#include"uniinc/meguse.inc"

      INTEGER IV,I

      V(IV     ) = X0
      V(IV +  1) = Y0
      V(IV +  2) = Z0
      V(IV +  3) = CONVP(1)
      V(IV +  4) = CONVP(2)
      V(IV +  5) = CONVP(3)

      IF (ENEDEP(22).GT.0.) THEN
         DO I = 1,3
            XAVE(I) = XAVE(I)/ENEDEP(22)
         ENDDO
         LONFL = LONFL/ENEDEP(22)
         LATFL = LATFL/ENEDEP(22)
      ENDIF

      V(IV +  6) = XAVE(1)
      V(IV +  7) = XAVE(2)
      V(IV +  8) = XAVE(3)

      V(IV +  9) = LONFL
      V(IV + 10) = LATFL

      IV = IV + 10

      RETURN
      END

C ============================================================= C

      SUBROUTINE LIXBUF(IV)

C ============================================================= C
C LiXe data buffer.                                             C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"
#include"uniinc/meglix.inc"

      INTEGER IV,I,J

C Inner Surface.
      V(IV) = FLOAT(NPMTI)

C Charges.
      DO I = 1,NPMTI
         V(IV + I) = PMTICH(I)
      ENDDO
      IV = IV + NPMTI

C Times (presently not included).
C      DO I = 1,NPMTI
C         V(IV + I) = PMTITI(I)
C      ENDDO
C      IV = IV + NPMTI

      IV = IV + 1

C Outer Surface.
      V(IV) = FLOAT(NPMTO)

C Charges.
      DO I = 1,NPMTO
         V(IV + I) = PMTOCH(I)
      ENDDO
      IV = IV + NPMTO

C Times (presently not included).
C      DO I = 1,NPMTO
C         V(IV + I) = PMTOTI(I)
C      ENDDO
C      IV = IV + NPMTO

       IV = IV + 1

C Up Cover.
      V(IV) = FLOAT(NPMTC)

C Charges.
      DO I = 1,NPMTC
         V(IV + I) = PMTUCH(I)
      ENDDO
      IV = IV + NPMTC

C Times (presently not included).
C      DO I = 1,NPMTC
C         V(IV + I) = PMTUTI(I)
C      ENDDO
C      IV = IV + NPMTC

      IV = IV + 1

C Down Cover.
      V(IV) = FLOAT(NPMTC)

C Charges.
      DO I = 1,NPMTC
         V(IV + I) = PMTDCH(I)
      ENDDO
      IV = IV + NPMTC

C Times (presently not included).
C      DO I = 1,NPMTC
C         V(IV + I) = PMTDTI(I)
C      ENDDO
C      IV = IV + NPMTC

       IV = IV + 1

C Left End. 
      V(IV) = FLOAT(NPMTE)

C Charges.
      DO I = 1,NPMTE
         V(IV + I) = PMTLCH(I)
      ENDDO
      IV = IV + NPMTE

C Times (presently not included).
C      DO I = 1,NPMTE
C         V(IV + I) = PMTLTI(I)
C      ENDDO
C      IV = IV + NPMTE

       IV = IV + 1

C Right End. 
      V(IV) = FLOAT(NPMTE)

C Charges.
      DO I = 1,NPMTE
         V(IV + I) = PMTRCH(I)
      ENDDO
      IV = IV + NPMTE

C Times (presently not included).
C      DO I = 1,NPMTE
C         V(IV + I) = PMTRTI(I)
C      ENDDO
C      IV = IV + NPMTE

C ============================================================= C
C PMT pulse shapes (presently not included).                    C
C ============================================================= C
C Inner Face.
C      DO I = 1,NPMTI
C        DO J = 1,NBADC
C          V(IV + I) = PULI(I,J)
C        ENDDO
C      ENDDO
C      IV = IV + NPMTI

C Outer Face.
C      DO I = 1,NPMTO
C        DO J = 1,NBADC
C          V(IV + I) = PULO(I,J)
C        ENDDO
C      ENDDO
C      IV = IV + NPMTO

C Up Cover.
C      DO I = 1,NPMTU
C        DO J = 1,NBADC
C          V(IV + I) = PULU(I,J)
C        ENDDO
C      ENDDO
C      IV = IV + NPMTU

C Down Cover.
C      DO I = 1,NPMTD
C        DO J = 1,NBADC
C          V(IV + I) = PULD(I,J)
C        ENDDO
C      ENDDO
C      IV = IV + NPMTD

C Left End.
C      DO I = 1,NPMTL
C        DO J = 1,NBADC
C          V(IV + I) = PULL(I,J)
C        ENDDO
C      ENDDO
C      IV = IV + NPMTL

C Right End.
C      DO I = 1,NPMTR
C        DO J = 1,NBADC
C          V(IV + I) = PULR(I,J)
C        ENDDO
C      ENDDO
C      IV = IV + NPMTR

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFBUF(IV)

C ============================================================= C
C Timing Counters data buffer.                                  C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"
#include"uniinc/megtof.inc"
#include"uniinc/megspe.inc"

      INTEGER IV,I,J

      V(IV) = FLOAT(NPHOD)

      IF (NPHOD.GT.0) THEN
         DO I = 1,NPHOD
            DO J = 1,8
               V(IV + 8*(I - 1) + J) = TCH(I,J)
            ENDDO
         ENDDO
         IV = IV + NPHOD*8
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEBUF(IV)

C ============================================================= C
C Spectrometer buffer.                                          C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"
#include"uniinc/megspe.inc"

CCCCC TEMPORANEO PER ROBERTO CCCCCCCCCCCCCCCCCC
#include"uniinc/megeve.inc"
      integer itime,itrk
      data itime/0/
      real ptot
CCCCC TEMPORANEO PER ROBERTO CCCCCCCCCCCCCCCCCC

      INTEGER IV,I,J

      V(IV) = FLOAT(NTRK1)

      IF (NTRK1.GT.0) THEN
         DO I = 1,NTRK1
            DO J = 1,7
               V(IV + 7*(I - 1) + J) = SPH(I,J)
            ENDDO
         ENDDO
         IV = IV + NTRK1*7
      ENDIF

CCCCC TEMPORANEO PER ROBERTO CCCCCCCCCCCCCCCCCC
      if (itime.eq.0) then
         itime = 1
         itrk = 0
         open(79,file = 'tracks.dat',status = 'unknown')
      endif
      itrk = itrk + 1
      if (itrk.gt.100) then 
         close(79)
      else
         ptot = sqrt(pinit(1,1)**2 + pinit(1,2)**2 + pinit(1,3)**2)
         do i = 1,ntrk1
            write(79,*) itrk,ptot,(sph(i,j),j = 1,3),sph(i,5)
         enddo
      endif
CCCCC TEMPORANEO PER ROBERTO CCCCCCCCCCCCCCCCCC

      RETURN
      END

C ================================================================= C

      SUBROUTINE MEGRES

C ================================================================= C
C Reset every output information.                                   C
C ================================================================= C

      IMPLICIT NONE

#include"uniinc/meglix.inc"
#include"uniinc/megspe.inc"
#include"uniinc/megtof.inc"
#include"uniinc/meguti.inc"

      INTEGER I,J

C LiXe calorimeter.
      DO I = 1,NPMTI
         PMTICH(I) = 0.
         PMTITI(I) = 0.
      ENDDO

      DO I = 1,NPMTO
         PMTOCH(I) = 0.
         PMTOTI(I) = 0.
      ENDDO

      DO I = 1,NPMTC
         PMTUCH(I) = 0.
         PMTUTI(I) = 0.
      ENDDO

      DO I = 1,NPMTC
         PMTDCH(I) = 0.
         PMTDTI(I) = 0.
      ENDDO

      DO I = 1,NPMTE
         PMTLCH(I) = 0.
         PMTLTI(I) = 0.
      ENDDO

      DO I = 1,NPMTE
         PMTRCH(I) = 0.
         PMTRTI(I) = 0.
      ENDDO

C Spectrometer.
      DO I = 1,MAXHSP
         DO J = 1,MSPEVA
            SPH(I,J) = 0.
         ENDDO
      ENDDO

C Timing counter.
      DO I = 1,MAXHTC
         DO J = 1,MTOFVA
            TCH(I,J) = 0.
         ENDDO
      ENDDO

      RETURN
      END

C ============================================================= C
C Data reading and decoding.                                    C
C ============================================================= C

C ============================================================= C

      SUBROUTINE MEGOUR(FILENAME,IRET)

C ============================================================= C
C Read unformatted MC general output.                           C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER IRET,IV,NV
      INTEGER ITIME
      DATA ITIME/0/
      CHARACTER*64 FILENAME,OLDFILE
      DATA OLDFILE/' '/

C Return code: IRET = 0: event ok, EOF not found;
C              IRET = 1:   "   "    "  found;
C              IRET = 2: bad event.
C              IRET = 3: error in opening file

      IF (ITIME.EQ.0) THEN
         OPEN(LUNIN,FILE = FILENAME,FORM = 'UNFORMATTED', 
     +        STATUS = 'OLD',ERR = 98)
         ITIME = 1
         WRITE(*,*) ' Opening file ',FILENAME
         OLDFILE = FILENAME
      ENDIF

C This is needed for multiple files.
      IF (OLDFILE.NE.FILENAME) THEN
         OPEN(LUNIN,FILE = FILENAME,FORM = 'UNFORMATTED', 
     +        STATUS = 'OLD',ERR = 98)         
         OLDFILE = FILENAME
      ENDIF
      
      IRET = 0
      READ(LUNIN,END = 100,ERR = 99) NV,(V(IV),IV = 1,NV)

C Good event: decode unformatted output.
      IF (IRET.EQ.0) CALL MEGOUD(NV,IRET)

C Problems in decoding ?
      IF (IRET.EQ.2) GOTO 99

C No problems !
      RETURN

 98   CONTINUE
      IRET = 3
      RETURN

 99   CONTINUE
      IRET = 2
      RETURN

C End of file reached.
 100  CONTINUE
      IRET = 1

      RETURN
      END

C ============================================================= C

      SUBROUTINE MEGOUD(NV,IRET)

C ============================================================= C
C Decode unformatted output.                                    C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER NV,IRET,NREAD
      INTEGER I,IV

C Header.
      IV = 1
      IDAT = INT(V(IV))

      IV = IV + 1
      EVNUMB = INT(V(IV))

C MC Data.
      IF (IDAT.EQ.0) THEN

         IV = IV + 1 
         IPOOK = INT(V(IV)) 

         IF (IPOOK.EQ.1) THEN
            DO I = 1,3
               PPO(I) = V(IV + I) 
               VPO(I) = V(IV + 3 + I) 
            ENDDO
            IV = IV + 6
         ENDIF
         
         IV = IV + 1
         IGAOK = INT(V(IV)) 
         
         IF (IGAOK.EQ.1) THEN
            DO I = 1,3
               PPHO(I) = V(IV + I)
               VPHO(I) = V(IV + 3 + I)
            ENDDO
            IV = IV + 6

         ELSE IF (IGAOK.EQ.2) THEN
            DO I = 1,3
               PPHO(I) = V(IV + I)
               VPHO(I) = V(IV + 3 + I)
            ENDDO
            IV = IV + 6

            DO I = 1,3
               PPHOPI(I) = V(IV + I)
               VPHOPI(I) = V(IV + 3 + I)
            ENDDO
            IV = IV + 6

         ENDIF
         
         IV = IV + 1

C Decode energy loss.
         CALL ENELOD(IV)

C ATTIVARLA PER LEGGERE I NUOVI FILES
         IV = IV + 1

C Decode gamma interaction point.
C TOGLIERE QUESTA ISTRUZIONE PER LEGGERE I VECCHI FILES (.OUT_ORSO_* etc.)
C INSERIRE QUESTA ISTRUZIONE PER LEGGERE I NUOVI FILES (.OUT etc.)
          CALL INTPOD(IV)
      ENDIF
      IV = IV + 1

C Decode LiXe buffer.      
      CALL LIXBD(IV)
      IV = IV + 1

C Decode Tof buffer.
      CALL TOFBD(IV)
      IV = IV + 1

C Decode Spectrometer buffer.
      CALL SPEBD(IV)

C Total number of read variables.
      NREAD = IV      

C File error.
      IF (NREAD.NE.NV) THEN
         WRITE(*,*) ' Error in reading file: NREAD = ',NREAD,' NV = ',NV
         IRET = 2
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE ENELOD(IV)

C ============================================================= C
C Decode energy loss.                                           C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER IV

      ELXE = V(IV     )
      EPTL = V(IV +  1)      
      EPMT = V(IV +  2)      
      EPVT = V(IV +  3)      
      ENIT = V(IV +  4)      
      EKAP = V(IV +  5)      
      EHET = V(IV +  6)
      EHEL = V(IV +  7)
      EG10 = V(IV +  8)
      EPTS = V(IV +  9)
      EALC = V(IV + 10) 
      EVCC = V(IV + 11)
      EPTT = V(IV + 12)                                         

      IV = IV + 12

      RETURN
      END

C ============================================================= C

      SUBROUTINE INTPOD(IV)

C ============================================================= C
C Decode gamma interaction and pair conversion point and center C
C of energy deposit.                                            C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER IV

      X0       = V(IV     )
      Y0       = V(IV +  1)      
      Z0       = V(IV +  2)      

      CONVP(1) = V(IV +  3)      
      CONVP(2) = V(IV +  4)      
      CONVP(3) = V(IV +  5)      

      XAVE(1)  = V(IV +  6)
      XAVE(2)  = V(IV +  7)
      XAVE(3)  = V(IV +  8)

      LONFL    = V(IV +  9)
      LATFL    = V(IV + 10)

      IV = IV + 10

      RETURN
      END

C ============================================================= C

      SUBROUTINE LIXBD(IV)

C ============================================================= C
C Decode LiXe buffer.                                           C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER I,J,IV

C Inner Surface.
      ANPI = INT(V(IV))

C Charges.
      DO I = 1,ANPI
         APCHI(I) = V(IV + I) 
      ENDDO
      IV = IV + ANPI

C Times (presently not included).
C      DO I = 1,ANPI
C         ATCHI(I) = V(IV + I) 
C      ENDDO
C      IV = IV + ANPI

      IV = IV + 1

C Outer Surface.
      ANPO = INT(V(IV))

      DO I = 1,ANPO
         APCHO(I) = V(IV + I) 
      ENDDO
      IV = IV + ANPO

C Times (presently not included).
C      DO I = 1,ANPO
C         ATCHO(I) = V(IV + I) 
C      ENDDO
C      IV = IV + ANPO

       IV = IV + 1

C Up Cover.
      ANPU = INT(V(IV))

      DO I = 1,ANPU
         APCHU(I) = V(IV + I) 
      ENDDO
      IV = IV + ANPU

C Times (presently not included).
C      DO I = 1,ANPU
C         ATCHU(I) = V(IV + I) 
C      ENDDO
C      IV = IV + ANPU

       IV = IV + 1

C Down Cover.
      ANPD = INT(V(IV))

C Charges.
      DO I = 1,ANPD
         APCHD(I) = V(IV + I) 
      ENDDO
      IV = IV + ANPD

C Times (presently not included).
C      DO I = 1,ANPD
C         ATCHD(I) = V(IV + I) 
C      ENDDO
C      IV = IV + ANPD

      IV = IV + 1

C Left End. 
      ANPL = INT(V(IV))

C Charges.
      DO I = 1,ANPL
         APCHL(I) = V(IV + I) 
      ENDDO
      IV = IV + ANPL

C Times (presently not included).
C      DO I = 1,ANPL
C         ATCHL(I) = V(IV + I) 
C      ENDDO
C      IV = IV + ANPL

      IV = IV + 1

C Right End. 
      ANPR = INT(V(IV))

C Charges.
      DO I = 1,ANPR
         APCHR(I) = V(IV + I) 
      ENDDO
      IV = IV + ANPR

C Times (presently not included).
C      DO I = 1,ANPR
C         ATCHR(I) = V(IV + I) 
C      ENDDO
C      IV = IV + ANPR

C ============================================================= C
C PMT pulse shapes (presently not included).                    C
C ============================================================= C
C Inner Face.
C      DO I = 1,ANPI
C        DO J = 1,NBADC
C          APULI(I,J) = V(IV + I)
C        ENDDO
C      ENDDO
C      IV = IV + ANPI

C Outer Face.
C      DO I = 1,ANPO
C        DO J = 1,NBADC
C          APULO(I,J) = V(IV + I)
C        ENDDO
C      ENDDO
C      IV = IV + ANPO

C Up Cover.
C      DO I = 1,ANPU
C        DO J = 1,NBADC
C          APULU(I,J) = V(IV + I)
C        ENDDO
C      ENDDO
C      IV = IV + ANPU

C Down Cover.
C      DO I = 1,ANPD
C        DO J = 1,NBADC
C          APULD(I,J) = V(IV + I)
C        ENDDO
C      ENDDO
C      IV = IV + ANPD

C Left End.
C      DO I = 1,ANPL
C        DO J = 1,NBADC
C          APULL(I,J) = V(IV + I)
C        ENDDO
C      ENDDO
C      IV = IV + ANPL

C Right End.
C      DO I = 1,ANPR
C        DO J = 1,NBADC
C          APULR(I,J) = V(IV + I)
C        ENDDO
C      ENDDO
C      IV = IV + ANPR

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFBD(IV)

C ============================================================= C
C Decode Tof buffer.                                            C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER I,IV,J

      NTOFEV = INT(V(IV))

      IF (NTOFEV.GT.0) THEN
         DO I = 1,NTOFEV
            DO J = 1,NTVA
               TOFEV(I,J) = V(IV + NTVA*(I - 1) + J) 
            ENDDO
         ENDDO
         IV = IV + NTOFEV*NTVA
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEBD(IV)

C ============================================================= C
C Decode Spectrometer buffer.                                   C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      INTEGER I,IV,J

      NSPEV = INT(V(IV))

      IF (NSPEV.GT.0) THEN
         DO I = 1,NSPEV
            DO J = 1,NSVA
               SPEV(I,J) = V(IV + NSVA*(I - 1) + J) 
            ENDDO
         ENDDO
         IV = IV + NSPEV*NSVA
      ENDIF

      RETURN
      END

C ================================================================= C
C Data analysis.                                                    C
C ================================================================= C

C ================================================================= C

      REAL FUNCTION RLANG(P)

C ================================================================= C
C Compute the relative angle between positron and photon.           C
C ================================================================= C

      IMPLICIT NONE

      REAL P(2,3)
      REAL VDOT
      REAL VPO(3),VPH(3)
      REAL SCAPR,SCAMO
      INTEGER I

      DO I = 1,3
         VPO(I) = P(1,I)
         VPH(I) = P(2,I)
      ENDDO

      SCAPR = VDOT(VPO,VPH,3)
      SCAMO = SQRT(VDOT(VPO,VPO,3)*VDOT(VPH,VPH,3))

      RLANG = ACOS(SCAPR/SCAMO)

      RETURN
      END

